@@ -4,8 +4,8 @@
  *	This file contains routines for converting file names betwen native
  *	and network form.
  *
- * Copyright (c) 1995-1998 Sun Microsystems, Inc.
- * Copyright (c) 1998-1999 Scriptics Corporation.
+ * Copyright © 1995-1998 Sun Microsystems, Inc.
+ * Copyright © 1998-1999 Scriptics Corporation.
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -26,8 +26,6 @@ TclPlatformType tclPlatform = TCL_PLATFORM_UNIX;
  * Prototypes for local procedures defined in this file:
  */
 
-static const char *	DoTildeSubst(Tcl_Interp *interp,
-			    const char *user, Tcl_DString *resultPtr);
 static const char *	ExtractWinRoot(const char *path,
 			    Tcl_DString *resultPtr, int offset,
 			    Tcl_PathType *typePtr);
@@ -41,6 +39,11 @@ static int		TclGlob(Tcl_Interp *interp, char *pattern,
 			    Tcl_Obj *pathPrefix, int globFlags,
 			    Tcl_GlobTypeData *types);
 
+/* Flag values used by TclGlob() */
+
+#define TCL_GLOBMODE_DIR	4
+#define TCL_GLOBMODE_TAILS	8
+
 /*
  * When there is no support for getting the block size of a file in a stat()
  * call, use this as a guess. Allow it to be overridden in the platform-
@@ -110,14 +113,14 @@ ExtractWinRoot(
 {
     int extended = 0;
 
-    if (   (path[0] == '/' || path[0] == '\\')
-	&& (path[1] == '/' || path[1] == '\\')
-	&& (path[2] == '?')
-	&& (path[3] == '/' || path[3] == '\\')) {
+    if (       (path[0] == '/' || path[0] == '\\')
+	    && (path[1] == '/' || path[1] == '\\')
+	    && (path[2] == '?')
+	    && (path[3] == '/' || path[3] == '\\')) {
 	extended = 1;
 	path = path + 4;
 	if (path[0] == 'U' && path[1] == 'N' && path[2] == 'C'
-	    && (path[3] == '/' || path[3] == '\\')) {
+		&& (path[3] == '/' || path[3] == '\\')) {
 	    extended = 2;
 	    path = path + 4;
 	}
@@ -365,12 +368,6 @@ Tcl_GetPathType(
  *	file). The exported function Tcl_FSGetPathType should be used by
  *	extensions.
  *
- *	Note that '~' paths are always considered TCL_PATH_ABSOLUTE, even
- *	though expanding the '~' could lead to any possible path type. This
- *	function should therefore be considered a low-level, string
- *	manipulation function only -- it doesn't actually do any expansion in
- *	making its determination.
- *
  * Results:
  *	Returns one of TCL_PATH_ABSOLUTE, TCL_PATH_RELATIVE, or
  *	TCL_PATH_VOLUME_RELATIVE.
@@ -384,78 +381,60 @@ Tcl_GetPathType(
 Tcl_PathType
 TclpGetNativePathType(
     Tcl_Obj *pathPtr,		/* Native path of interest */
-    int *driveNameLengthPtr,	/* Returns length of drive, if non-NULL and
-				 * path was absolute */
+    Tcl_Size *driveNameLengthPtr, /* Returns length of drive, if non-NULL and
+				   * path was absolute */
     Tcl_Obj **driveNameRef)
 {
     Tcl_PathType type = TCL_PATH_ABSOLUTE;
-    int pathLen;
-    const char *path = TclGetStringFromObj(pathPtr, &pathLen);
+    const char *path = TclGetString(pathPtr);
+
+    switch (tclPlatform) {
+    case TCL_PLATFORM_UNIX: {
+	const char *origPath = path;
 
-    if (path[0] == '~') {
 	/*
-	 * This case is common to all platforms. Paths that begin with ~ are
-	 * absolute.
+	 * Paths that begin with / are absolute.
 	 */
 
-	if (driveNameLengthPtr != NULL) {
-	    const char *end = path + 1;
-	    while ((*end != '\0') && (*end != '/')) {
-		end++;
-	    }
-	    *driveNameLengthPtr = end - path;
-	}
-    } else {
-	switch (tclPlatform) {
-	case TCL_PLATFORM_UNIX: {
-	    const char *origPath = path;
-
+	if (path[0] == '/') {
+	    ++path;
 	    /*
-	     * Paths that begin with / are absolute.
+	     * Check for "//" network path prefix
 	     */
-
-	    if (path[0] == '/') {
-		++path;
-#if defined(__CYGWIN__) || defined(__QNX__)
+	    if ((*path == '/') && path[1] && (path[1] != '/')) {
+		path += 2;
+		while (*path && *path != '/') {
+		    ++path;
+		}
+	    }
+	    if (driveNameLengthPtr != NULL) {
 		/*
-		 * Check for "//" network path prefix
+		 * We need this addition in case the "//" code was used.
 		 */
-		if ((*path == '/') && path[1] && (path[1] != '/')) {
-		    path += 2;
-		    while (*path && *path != '/') {
-			++path;
-		    }
-		}
-#endif
-		if (driveNameLengthPtr != NULL) {
-		    /*
-		     * We need this addition in case the "//" code was used.
-		     */
 
-		    *driveNameLengthPtr = (path - origPath);
-		}
-	    } else {
-		type = TCL_PATH_RELATIVE;
+		*driveNameLengthPtr = (path - origPath);
 	    }
-	    break;
+	} else {
+	    type = TCL_PATH_RELATIVE;
 	}
-	case TCL_PLATFORM_WINDOWS: {
-	    Tcl_DString ds;
-	    const char *rootEnd;
+	break;
+    }
+    case TCL_PLATFORM_WINDOWS: {
+	Tcl_DString ds;
+	const char *rootEnd;
 
-	    Tcl_DStringInit(&ds);
-	    rootEnd = ExtractWinRoot(path, &ds, 0, &type);
-	    if ((rootEnd != path) && (driveNameLengthPtr != NULL)) {
-		*driveNameLengthPtr = rootEnd - path;
-		if (driveNameRef != NULL) {
-		    *driveNameRef = TclDStringToObj(&ds);
-		    Tcl_IncrRefCount(*driveNameRef);
-		}
+	Tcl_DStringInit(&ds);
+	rootEnd = ExtractWinRoot(path, &ds, 0, &type);
+	if ((rootEnd != path) && (driveNameLengthPtr != NULL)) {
+	    *driveNameLengthPtr = rootEnd - path;
+	    if (driveNameRef != NULL) {
+		*driveNameRef = Tcl_DStringToObj(&ds);
+		Tcl_IncrRefCount(*driveNameRef);
 	    }
-	    Tcl_DStringFree(&ds);
-	    break;
-	}
 	}
+	Tcl_DStringFree(&ds);
+	break;
+    }
     }
     return type;
 }
@@ -486,7 +465,7 @@ TclpGetNativePathType(
 Tcl_Obj *
 TclpNativeSplitPath(
     Tcl_Obj *pathPtr,		/* Path to split. */
-    int *lenPtr)		/* int to store number of path elements. */
+    Tcl_Size *lenPtr)		/* int to store number of path elements. */
 {
     Tcl_Obj *resultPtr = NULL;	/* Needed only to prevent gcc warnings. */
 
@@ -529,7 +508,7 @@ TclpNativeSplitPath(
  *	*argcPtr will get filled in with the number of valid elements in the
  *	array. A single block of memory is dynamically allocated to hold both
  *	the argv array and a copy of the path elements. The caller must
- *	eventually free this memory by calling ckfree() on *argvPtr. Note:
+ *	eventually free this memory by calling Tcl_Free() on *argvPtr. Note:
  *	*argvPtr and *argcPtr are only modified if the procedure returns
  *	normally.
  *
@@ -539,17 +518,18 @@ TclpNativeSplitPath(
  *----------------------------------------------------------------------
  */
 
+#undef Tcl_SplitPath
 void
 Tcl_SplitPath(
     const char *path,		/* Pointer to string containing a path. */
-    int *argcPtr,		/* Pointer to location to fill in with the
+    Tcl_Size *argcPtr,		/* Pointer to location to fill in with the
 				 * number of elements in the path. */
     const char ***argvPtr)	/* Pointer to place to store pointer to array
 				 * of pointers to path elements. */
 {
     Tcl_Obj *resultPtr = NULL;	/* Needed only to prevent gcc warnings. */
     Tcl_Obj *tmpPtr, *eltPtr;
-    int i, size, len;
+    Tcl_Size i, size, len;
     char *p;
     const char *str;
 
@@ -570,7 +550,7 @@ Tcl_SplitPath(
     size = 1;
     for (i = 0; i < *argcPtr; i++) {
 	Tcl_ListObjIndex(NULL, resultPtr, i, &eltPtr);
-	TclGetStringFromObj(eltPtr, &len);
+	(void)TclGetStringFromObj(eltPtr, &len);
 	size += len + 1;
     }
 
@@ -579,7 +559,7 @@ Tcl_SplitPath(
      * plus the argv pointers and the terminating NULL pointer.
      */
 
-    *argvPtr = (const char **)ckalloc(
+    *argvPtr = (const char **)Tcl_Alloc(
 	    ((((*argcPtr) + 1) * sizeof(char *)) + size));
 
     /*
@@ -603,7 +583,7 @@ Tcl_SplitPath(
 
     for (i = 0; i < *argcPtr; i++) {
 	(*argvPtr)[i] = p;
-	for (; *(p++)!='\0'; );
+	while (*(p++) != '\0');
     }
     (*argvPtr)[i] = NULL;
 
@@ -635,7 +615,7 @@ static Tcl_Obj *
 SplitUnixPath(
     const char *path)		/* Pointer to string containing a path. */
 {
-    int length;
+    size_t length;
     const char *origPath = path, *elementStart;
     Tcl_Obj *result;
 
@@ -647,7 +627,6 @@ SplitUnixPath(
     if (*path == '/') {
 	Tcl_Obj *rootElt;
 	++path;
-#if defined(__CYGWIN__) || defined(__QNX__)
 	/*
 	 * Check for "//" network path prefix
 	 */
@@ -657,7 +636,6 @@ SplitUnixPath(
 		++path;
 	    }
 	}
-#endif
 	rootElt = Tcl_NewStringObj(origPath, path - origPath);
 	Tcl_ListObjAppendElement(NULL, result, rootElt);
 	while (*path == '/') {
@@ -666,8 +644,7 @@ SplitUnixPath(
     }
 
     /*
-     * Split on slashes. Embedded elements that start with tilde will be
-     * prefixed with "./" so they are not affected by tilde substitution.
+     * Split on slashes.
      */
 
     for (;;) {
@@ -678,12 +655,7 @@ SplitUnixPath(
 	length = path - elementStart;
 	if (length > 0) {
 	    Tcl_Obj *nextElt;
-	    if ((elementStart[0] == '~') && (elementStart != origPath)) {
-		TclNewLiteralStringObj(nextElt, "./");
-		Tcl_AppendToObj(nextElt, elementStart, length);
-	    } else {
-		nextElt = Tcl_NewStringObj(elementStart, length);
-	    }
+	    nextElt = Tcl_NewStringObj(elementStart, length);
 	    Tcl_ListObjAppendElement(NULL, result, nextElt);
 	}
 	if (*path++ == '\0') {
@@ -714,7 +686,7 @@ static Tcl_Obj *
 SplitWinPath(
     const char *path)		/* Pointer to string containing a path. */
 {
-    int length;
+    size_t length;
     const char *p, *elementStart;
     Tcl_PathType type = TCL_PATH_ABSOLUTE;
     Tcl_DString buf;
@@ -729,14 +701,12 @@ SplitWinPath(
      */
 
     if (p != path) {
-	Tcl_ListObjAppendElement(NULL, result, TclDStringToObj(&buf));
+	Tcl_ListObjAppendElement(NULL, result, Tcl_DStringToObj(&buf));
     }
     Tcl_DStringFree(&buf);
 
     /*
-     * Split on slashes. Embedded elements that start with tilde or a drive
-     * letter will be prefixed with "./" so they are not affected by tilde
-     * substitution.
+     * Split on slashes.
      */
 
     do {
@@ -747,9 +717,9 @@ SplitWinPath(
 	length = p - elementStart;
 	if (length > 0) {
 	    Tcl_Obj *nextElt;
-	    if ((elementStart != path) && ((elementStart[0] == '~')
-		    || (isalpha(UCHAR(elementStart[0]))
-			&& elementStart[1] == ':'))) {
+	    if ((elementStart != path) &&
+		    isalpha(UCHAR(elementStart[0])) &&
+		    (elementStart[1] == ':')) {
 		TclNewLiteralStringObj(nextElt, "./");
 		Tcl_AppendToObj(nextElt, elementStart, length);
 	    } else {
@@ -788,7 +758,7 @@ SplitWinPath(
 Tcl_Obj *
 Tcl_FSJoinToPath(
     Tcl_Obj *pathPtr,		/* Valid path or NULL. */
-    int objc,			/* Number of array elements to join */
+    Tcl_Size objc,		/* Number of array elements to join */
     Tcl_Obj *const objv[])	/* Path elements to join. */
 {
     if (pathPtr == NULL) {
@@ -804,13 +774,13 @@ Tcl_FSJoinToPath(
 	pair[1] = objv[0];
 	return TclJoinPath(2, pair, 0);
     } else {
-	int elemc = objc + 1;
-	Tcl_Obj *ret, **elemv = (Tcl_Obj**)ckalloc(elemc*sizeof(Tcl_Obj *));
+	Tcl_Size elemc = objc + 1;
+	Tcl_Obj *ret, **elemv = (Tcl_Obj**)Tcl_Alloc(elemc*sizeof(Tcl_Obj *));
 
 	elemv[0] = pathPtr;
 	memcpy(elemv+1, objv, objc*sizeof(Tcl_Obj *));
 	ret = TclJoinPath(elemc, elemv, 0);
-	ckfree(elemv);
+	Tcl_Free(elemv);
 	return ret;
     }
 }
@@ -836,7 +806,8 @@ TclpNativeJoinPath(
     Tcl_Obj *prefix,
     const char *joining)
 {
-    int length, needsSep;
+    int needsSep;
+    Tcl_Size length;
     char *dest;
     const char *p;
     const char *start;
@@ -844,16 +815,16 @@ TclpNativeJoinPath(
     start = TclGetStringFromObj(prefix, &length);
 
     /*
-     * Remove the ./ from tilde prefixed elements, and drive-letter prefixed
+     * Remove the ./ from drive-letter prefixed
      * elements on Windows, unless it is the first component.
      */
 
     p = joining;
 
     if (length != 0) {
-	if ((p[0] == '.') && (p[1] == '/') && ((p[2] == '~')
-		|| (tclPlatform==TCL_PLATFORM_WINDOWS && isalpha(UCHAR(p[2]))
-		&& (p[3] == ':')))) {
+	if ((p[0] == '.') && (p[1] == '/') &&
+		(tclPlatform==TCL_PLATFORM_WINDOWS) && isalpha(UCHAR(p[2]))
+		&& (p[3] == ':')) {
 	    p += 2;
 	}
     }
@@ -869,7 +840,7 @@ TclpNativeJoinPath(
 
 	if (length > 0 && (start[length-1] != '/')) {
 	    Tcl_AppendToObj(prefix, "/", 1);
-	    TclGetStringFromObj(prefix, &length);
+	    (void)TclGetStringFromObj(prefix, &length);
 	}
 	needsSep = 0;
 
@@ -877,7 +848,7 @@ TclpNativeJoinPath(
 	 * Append the element, eliminating duplicate and trailing slashes.
 	 */
 
-	Tcl_SetObjLength(prefix, length + (int) strlen(p));
+	Tcl_SetObjLength(prefix, length + strlen(p));
 
 	dest = TclGetString(prefix) + length;
 	for (; *p != '\0'; p++) {
@@ -905,7 +876,7 @@ TclpNativeJoinPath(
 	if ((length > 0) &&
 		(start[length-1] != '/') && (start[length-1] != ':')) {
 	    Tcl_AppendToObj(prefix, "/", 1);
-	    TclGetStringFromObj(prefix, &length);
+	    (void)TclGetStringFromObj(prefix, &length);
 	}
 	needsSep = 0;
 
@@ -956,11 +927,11 @@ TclpNativeJoinPath(
 
 char *
 Tcl_JoinPath(
-    int argc,
+    Tcl_Size argc,
     const char *const *argv,
     Tcl_DString *resultPtr)	/* Pointer to previously initialized DString */
 {
-    int i, len;
+    Tcl_Size i, len;
     Tcl_Obj *listObj;
     Tcl_Obj *resultObj;
     const char *resultStr;
@@ -1005,19 +976,15 @@ Tcl_JoinPath(
  * Tcl_TranslateFileName --
  *
  *	Converts a file name into a form usable by the native system
- *	interfaces. If the name starts with a tilde, it will produce a name
- *	where the tilde and following characters have been replaced by the
- *	home directory location for the named user.
+ *	interfaces.
  *
  * Results:
- *	The return value is a pointer to a string containing the name after
- *	tilde substitution. If there was no tilde substitution, the return
- *	value is a pointer to a copy of the original string. If there was an
+ *	The return value is a pointer to a string containing the name.
+ *      This may either be the name pointer passed in or space allocated in
+ *      bufferPtr. In all cases, if the return value is not NULL, the caller
+ *	must call Tcl_DStringFree() to free the space. If there was an
  *	error in processing the name, then an error message is left in the
  *	interp's result (if interp was not NULL) and the return value is NULL.
- *	Space for the return value is allocated in bufferPtr; the caller must
- *	call Tcl_DStringFree() to free the space if the return value was not
- *	NULL.
  *
  * Side effects:
  *	None.
@@ -1034,7 +1001,7 @@ Tcl_TranslateFileName(
 				 * "~<user>" (to indicate any user's home
 				 * directory). */
     Tcl_DString *bufferPtr)	/* Uninitialized or free DString filled with
-				 * name after tilde substitution. */
+				 * name. */
 {
     Tcl_Obj *path = Tcl_NewStringObj(name, -1);
     Tcl_Obj *transPtr;
@@ -1126,65 +1093,6 @@ TclGetExtension(
     return p;
 }
 
-/*
- *----------------------------------------------------------------------
- *
- * DoTildeSubst --
- *
- *	Given a string following a tilde, this routine returns the
- *	corresponding home directory.
- *
- * Results:
- *	The result is a pointer to a static string containing the home
- *	directory in native format. If there was an error in processing the
- *	substitution, then an error message is left in the interp's result and
- *	the return value is NULL. On success, the results are appended to
- *	resultPtr, and the contents of resultPtr are returned.
- *
- * Side effects:
- *	Information may be left in resultPtr.
- *
- *----------------------------------------------------------------------
- */
-
-static const char *
-DoTildeSubst(
-    Tcl_Interp *interp,		/* Interpreter in which to store error message
-				 * (if necessary). */
-    const char *user,		/* Name of user whose home directory should be
-				 * substituted, or "" for current user. */
-    Tcl_DString *resultPtr)	/* Initialized DString filled with name after
-				 * tilde substitution. */
-{
-    const char *dir;
-
-    if (*user == '\0') {
-	Tcl_DString dirString;
-
-	dir = TclGetEnv("HOME", &dirString);
-	if (dir == NULL) {
-	    if (interp) {
-		Tcl_SetObjResult(interp, Tcl_NewStringObj(
-			"couldn't find HOME environment "
-			"variable to expand path", -1));
-		Tcl_SetErrorCode(interp, "TCL", "FILENAME", "NO_HOME", (char *)NULL);
-	    }
-	    return NULL;
-	}
-	Tcl_JoinPath(1, &dir, resultPtr);
-	Tcl_DStringFree(&dirString);
-    } else if (TclpGetUserHome(user, resultPtr) == NULL) {
-	if (interp) {
-	    Tcl_ResetResult(interp);
-	    Tcl_SetObjResult(interp, Tcl_ObjPrintf(
-		    "user \"%s\" doesn't exist", user));
-	    Tcl_SetErrorCode(interp, "TCL", "LOOKUP", "USER", user, (char *)NULL);
-	}
-	return NULL;
-    }
-    return Tcl_DStringValue(resultPtr);
-}
-
 /*
  *----------------------------------------------------------------------
  *
@@ -1204,12 +1112,13 @@ DoTildeSubst(
 
 int
 Tcl_GlobObjCmd(
-    ClientData dummy,		/* Not used. */
+    TCL_UNUSED(void *),
     Tcl_Interp *interp,		/* Current interpreter. */
     int objc,			/* Number of arguments. */
     Tcl_Obj *const objv[])	/* Argument objects. */
 {
-    int index, i, globFlags, length, join, dir, result;
+    int i, globFlags, join, dir, result;
+    Tcl_Size length;
     char *string;
     const char *separators;
     Tcl_Obj *typePtr, *look;
@@ -1222,19 +1131,18 @@ Tcl_GlobObjCmd(
     enum globOptionsEnum {
 	GLOB_DIR, GLOB_JOIN, GLOB_NOCOMPLAIN, GLOB_PATH, GLOB_TAILS,
 	GLOB_TYPE, GLOB_LAST
-    };
+    } index;
     enum pathDirOptions {PATH_NONE = -1 , PATH_GENERAL = 0, PATH_DIR = 1};
     Tcl_GlobTypeData *globTypes = NULL;
-    (void)dummy;
 
     globFlags = 0;
     join = 0;
     dir = PATH_NONE;
     typePtr = NULL;
     for (i = 1; i < objc; i++) {
-	if (Tcl_GetIndexFromObj(interp, objv[i], options, "option", 0,
-		&index) != TCL_OK) {
-	    string = TclGetStringFromObj(objv[i], &length);
+	if (Tcl_GetIndexFromObj(interp, objv[i], options,
+		"option", 0, &index) != TCL_OK) {
+	    string = TclGetString(objv[i]);
 	    if (string[0] == '-') {
 		/*
 		 * It looks like the command contains an option so signal an
@@ -1253,15 +1161,18 @@ Tcl_GlobObjCmd(
 	    }
 	}
 
-	switch ((enum globOptionsEnum) index) {
+	switch (index) {
 	case GLOB_NOCOMPLAIN:			/* -nocomplain */
-	    globFlags |= TCL_GLOBMODE_NO_COMPLAIN;
+	    /*
+	     * Do nothing; This is normal operations in Tcl 9.
+	     * Keep accepting as a no-op option to accommodate old scripts.
+	     */
 	    break;
 	case GLOB_DIR:				/* -dir */
 	    if (i == (objc-1)) {
 		Tcl_SetObjResult(interp, Tcl_NewStringObj(
 			"missing argument to \"-directory\"", -1));
-		Tcl_SetErrorCode(interp, "TCL", "ARGUMENT", "MISSING", (char *)NULL);
+		Tcl_SetErrorCode(interp, "TCL", "ARGUMENT", "MISSING", (void *)NULL);
 		return TCL_ERROR;
 	    }
 	    if (dir != PATH_NONE) {
@@ -1271,7 +1182,7 @@ Tcl_GlobObjCmd(
 			    : "\"-directory\" cannot be used with \"-path\"",
 			-1));
 		Tcl_SetErrorCode(interp, "TCL", "OPERATION", "GLOB",
-			"BADOPTIONCOMBINATION", (char *)NULL);
+			"BADOPTIONCOMBINATION", (void *)NULL);
 		return TCL_ERROR;
 	    }
 	    dir = PATH_DIR;
@@ -1289,7 +1200,7 @@ Tcl_GlobObjCmd(
 	    if (i == (objc-1)) {
 		Tcl_SetObjResult(interp, Tcl_NewStringObj(
 			"missing argument to \"-path\"", -1));
-		Tcl_SetErrorCode(interp, "TCL", "ARGUMENT", "MISSING", (char *)NULL);
+		Tcl_SetErrorCode(interp, "TCL", "ARGUMENT", "MISSING", (void *)NULL);
 		return TCL_ERROR;
 	    }
 	    if (dir != PATH_NONE) {
@@ -1299,7 +1210,7 @@ Tcl_GlobObjCmd(
 			    : "\"-path\" cannot be used with \"-dictionary\"",
 			-1));
 		Tcl_SetErrorCode(interp, "TCL", "OPERATION", "GLOB",
-			"BADOPTIONCOMBINATION", (char *)NULL);
+			"BADOPTIONCOMBINATION", (void *)NULL);
 		return TCL_ERROR;
 	    }
 	    dir = PATH_GENERAL;
@@ -1310,7 +1221,7 @@ Tcl_GlobObjCmd(
 	    if (i == (objc-1)) {
 		Tcl_SetObjResult(interp, Tcl_NewStringObj(
 			"missing argument to \"-types\"", -1));
-		Tcl_SetErrorCode(interp, "TCL", "ARGUMENT", "MISSING", (char *)NULL);
+		Tcl_SetErrorCode(interp, "TCL", "ARGUMENT", "MISSING", (void *)NULL);
 		return TCL_ERROR;
 	    }
 	    typePtr = objv[i+1];
@@ -1331,7 +1242,7 @@ Tcl_GlobObjCmd(
 		"\"-tails\" must be used with either "
 		"\"-directory\" or \"-path\"", -1));
 	Tcl_SetErrorCode(interp, "TCL", "OPERATION", "GLOB",
-		"BADOPTIONCOMBINATION", (char *)NULL);
+		"BADOPTIONCOMBINATION", (void *)NULL);
 	return TCL_ERROR;
     }
 
@@ -1346,7 +1257,7 @@ Tcl_GlobObjCmd(
     }
 
     if (dir == PATH_GENERAL) {
-	int pathlength;
+	Tcl_Size pathlength;
 	const char *last;
 	const char *first = TclGetStringFromObj(pathOrDir,&pathlength);
 
@@ -1356,7 +1267,7 @@ Tcl_GlobObjCmd(
 
 	last = first + pathlength;
 	for (; last != first; last--) {
-	    if (strchr(separators, *(last-1)) != NULL) {
+	    if (strchr(separators, last[-1]) != NULL) {
 		break;
 	    }
 	}
@@ -1395,11 +1306,15 @@ Tcl_GlobObjCmd(
 		 * We must ensure that we haven't cut off too much, and turned
 		 * a valid path like '/' or 'C:/' into an incorrect path like
 		 * '' or 'C:'. The way we do this is to add a separator if
-		 * there are none presently in the prefix.
+		 * there are none presently in the prefix. Similar treatment
+		 * for the zipfs volume.
 		 */
 
-		if (strpbrk(TclGetString(pathOrDir), "\\/") == NULL) {
+		const char *temp = TclGetString(pathOrDir);
+		if (strpbrk(temp, "\\/") == NULL) {
 		    Tcl_AppendToObj(pathOrDir, last-1, 1);
+		} else if (!strcmp(temp, "//zipfs:")) {
+		    Tcl_AppendToObj(pathOrDir, "/", 1);
 		}
 	    }
 
@@ -1437,7 +1352,7 @@ Tcl_GlobObjCmd(
 	 */
 
 	TclListObjLength(interp, typePtr, &length);
-	if (length <= 0) {
+	if (length == 0) {
 	    goto skipTypes;
 	}
 	globTypes = (Tcl_GlobTypeData *)TclStackAlloc(interp, sizeof(Tcl_GlobTypeData));
@@ -1446,8 +1361,8 @@ Tcl_GlobObjCmd(
 	globTypes->macType = NULL;
 	globTypes->macCreator = NULL;
 
-	while (--length >= 0) {
-	    int len;
+	while (length-- > 0) {
+	    Tcl_Size len;
 	    const char *str;
 
 	    Tcl_ListObjIndex(interp, typePtr, length, &look);
@@ -1505,9 +1420,10 @@ Tcl_GlobObjCmd(
 
 	    } else {
 		Tcl_Obj *item;
+		Tcl_Size llen;
 
-		if ((TclListObjLength(NULL, look, &len) == TCL_OK)
-			&& (len == 3)) {
+		if ((TclListObjLength(NULL, look, &llen) == TCL_OK)
+			&& (llen == 3)) {
 		    Tcl_ListObjIndex(interp, look, 0, &item);
 		    if (!strcmp("macintosh", TclGetString(item))) {
 			Tcl_ListObjIndex(interp, look, 1, &item);
@@ -1539,8 +1455,8 @@ Tcl_GlobObjCmd(
 	    badTypesArg:
 		Tcl_SetObjResult(interp, Tcl_ObjPrintf(
 			"bad argument to \"-types\": %s",
-			Tcl_GetString(look)));
-		Tcl_SetErrorCode(interp, "TCL", "ARGUMENT", "BAD", (char *)NULL);
+			TclGetString(look)));
+		Tcl_SetErrorCode(interp, "TCL", "ARGUMENT", "BAD", (void *)NULL);
 		result = TCL_ERROR;
 		join = 0;
 		goto endOfGlob;
@@ -1550,7 +1466,7 @@ Tcl_GlobObjCmd(
 			"only one MacOS type or creator argument"
 			" to \"-types\" allowed", -1));
 		result = TCL_ERROR;
-		Tcl_SetErrorCode(interp, "TCL", "ARGUMENT", "BAD", (char *)NULL);
+		Tcl_SetErrorCode(interp, "TCL", "ARGUMENT", "BAD", (void *)NULL);
 		join = 0;
 		goto endOfGlob;
 	    }
@@ -1612,41 +1528,6 @@ Tcl_GlobObjCmd(
 	}
     }
 
-    if ((globFlags & TCL_GLOBMODE_NO_COMPLAIN) == 0) {
-	if (TclListObjLength(interp, Tcl_GetObjResult(interp),
-		&length) != TCL_OK) {
-	    /*
-	     * This should never happen. Maybe we should be more dramatic.
-	     */
-
-	    result = TCL_ERROR;
-	    goto endOfGlob;
-	}
-
-	if (length == 0) {
-	    Tcl_Obj *errorMsg =
-		    Tcl_ObjPrintf("no files matched glob pattern%s \"",
-			    (join || (objc == 1)) ? "" : "s");
-
-	    if (join) {
-		Tcl_AppendToObj(errorMsg, Tcl_DStringValue(&prefix), -1);
-	    } else {
-		const char *sep = "";
-
-		for (i = 0; i < objc; i++) {
-		    Tcl_AppendPrintfToObj(errorMsg, "%s%s",
-			    sep, TclGetString(objv[i]));
-		    sep = " ";
-		}
-	    }
-	    Tcl_AppendToObj(errorMsg, "\"", -1);
-	    Tcl_SetObjResult(interp, errorMsg);
-	    Tcl_SetErrorCode(interp, "TCL", "OPERATION", "GLOB", "NOMATCH",
-		    (char *)NULL);
-	    result = TCL_ERROR;
-	}
-    }
-
   endOfGlob:
     if (join || (dir == PATH_GENERAL)) {
 	Tcl_DStringFree(&prefix);
@@ -1671,8 +1552,7 @@ Tcl_GlobObjCmd(
  *
  * TclGlob --
  *
- *	Sets the separator string based on the platform, performs tilde
- *	substitution, and calls DoGlob.
+ *	Sets the separator string based on the platform	and calls DoGlob.
  *
  *	The interpreter's result, on entry to this function, must be a valid
  *	Tcl list (e.g. it could be empty), since we will lappend any new
@@ -1708,8 +1588,7 @@ TclGlob(
 				 * NULL. */
 {
     const char *separators;
-    const char *head;
-    char *tail, *start;
+    char *tail;
     int result;
     Tcl_Obj *filenamesObj, *savedResultObj;
 
@@ -1723,59 +1602,10 @@ TclGlob(
 	break;
     }
 
-    if (pathPrefix == NULL) {
-	char c;
-	Tcl_DString buffer;
-	Tcl_DStringInit(&buffer);
-
-	start = pattern;
-
-	/*
-	 * Perform tilde substitution, if needed.
-	 */
-
-	if (start[0] == '~') {
-	    /*
-	     * Find the first path separator after the tilde.
-	     */
-
-	    for (tail = start; *tail != '\0'; tail++) {
-		if (*tail == '\\') {
-		    if (strchr(separators, tail[1]) != NULL) {
-			break;
-		    }
-		} else if (strchr(separators, *tail) != NULL) {
-		    break;
-		}
-	    }
-
-	    /*
-	     * Determine the home directory for the specified user.
-	     */
-
-	    c = *tail;
-	    *tail = '\0';
-	    head = DoTildeSubst(interp, start+1, &buffer);
-	    *tail = c;
-	    if (head == NULL) {
-		return TCL_ERROR;
-	    }
-	    if (head != Tcl_DStringValue(&buffer)) {
-		Tcl_DStringAppend(&buffer, head, -1);
-	    }
-	    pathPrefix = TclDStringToObj(&buffer);
-	    Tcl_IncrRefCount(pathPrefix);
-	    globFlags |= TCL_GLOBMODE_DIR;
-	    if (c != '\0') {
-		tail++;
-	    }
-	} else {
-	    tail = pattern;
-	}
-    } else {
+    if (pathPrefix != NULL) {
 	Tcl_IncrRefCount(pathPrefix);
-	tail = pattern;
     }
+    tail = pattern;
 
     /*
      * Handling empty path prefixes with glob patterns like 'C:' or
@@ -1814,7 +1644,7 @@ TclGlob(
 	    Tcl_IncrRefCount(pathPrefix);
 	} else if (pathPrefix == NULL && (tail[0] == '/'
 		|| (tail[0] == '\\' && tail[1] == '\\'))) {
-	    int driveNameLen;
+	    Tcl_Size driveNameLen;
 	    Tcl_Obj *driveName;
 	    Tcl_Obj *temp = Tcl_NewStringObj(tail, -1);
 	    Tcl_IncrRefCount(temp);
@@ -1869,11 +1699,7 @@ TclGlob(
 	separators = "/\\";
 
     } else if (tclPlatform == TCL_PLATFORM_UNIX) {
-	if (pathPrefix == NULL && tail[0] == '/'
-#if defined(__CYGWIN__) || defined(__QNX__)
-		&& tail[1] != '/'
-#endif
-	) {
+	if (pathPrefix == NULL && tail[0] == '/' && tail[1] != '/') {
 	    pathPrefix = Tcl_NewStringObj(tail, 1);
 	    tail++;
 	    Tcl_IncrRefCount(pathPrefix);
@@ -1886,9 +1712,9 @@ TclGlob(
      */
 
     if (pathPrefix == NULL) {
-	int driveNameLen;
+	Tcl_Size driveNameLen;
 	Tcl_Obj *driveName;
-	if (TclFSNonnativePathType(tail, (int) strlen(tail), NULL,
+	if (TclFSNonnativePathType(tail, strlen(tail), NULL,
 		&driveNameLen, &driveName) == TCL_PATH_ABSOLUTE) {
 	    pathPrefix = driveName;
 	    tail += driveNameLen;
@@ -1971,9 +1797,9 @@ TclGlob(
      */
 
     if (globFlags & TCL_GLOBMODE_TAILS) {
-	int objc, i;
+	Tcl_Size objc, i;
 	Tcl_Obj **objv;
-	int prefixLen;
+	Tcl_Size prefixLen;
 	const char *pre;
 
 	/*
@@ -2001,7 +1827,7 @@ TclGlob(
 
 	TclListObjGetElements(NULL, filenamesObj, &objc, &objv);
 	for (i = 0; i< objc; i++) {
-	    int len;
+	    Tcl_Size len;
 	    const char *oldStr = TclGetStringFromObj(objv[i], &len);
 	    Tcl_Obj *elem;
 
@@ -2211,14 +2037,14 @@ DoGlob(
 	    Tcl_SetObjResult(interp, Tcl_NewStringObj(
 		    "unmatched open-brace in file name", -1));
 	    Tcl_SetErrorCode(interp, "TCL", "OPERATION", "GLOB", "BALANCE",
-		    (char *)NULL);
+		    (void *)NULL);
 	    return TCL_ERROR;
 
 	} else if (*p == '}') {
 	    Tcl_SetObjResult(interp, Tcl_NewStringObj(
 		    "unmatched close-brace in file name", -1));
 	    Tcl_SetErrorCode(interp, "TCL", "OPERATION", "GLOB", "BALANCE",
-		    (char *)NULL);
+		    (void *)NULL);
 	    return TCL_ERROR;
 	}
     }
@@ -2323,7 +2149,7 @@ DoGlob(
 		pattern, &dirOnly);
 	*p = save;
 	if (result == TCL_OK) {
-	    int subdirc, i, repair = -1;
+	    Tcl_Size i, subdirc, repair = -1;
 	    Tcl_Obj **subdirv;
 
 	    result = TclListObjGetElements(interp, subdirsPtr,
@@ -2331,24 +2157,17 @@ DoGlob(
 	    for (i=0; result==TCL_OK && i<subdirc; i++) {
 		Tcl_Obj *copy = NULL;
 
-		if (pathPtr == NULL && TclGetString(subdirv[i])[0] == '~') {
-		    TclListObjLength(NULL, matchesObj, &repair);
-		    copy = subdirv[i];
-		    subdirv[i] = Tcl_NewStringObj("./", 2);
-		    Tcl_AppendObjToObj(subdirv[i], copy);
-		    Tcl_IncrRefCount(subdirv[i]);
-		}
 		result = DoGlob(interp, matchesObj, separators, subdirv[i],
 			1, p+1, types);
 		if (copy) {
-		    int end;
+		    Tcl_Size end;
 
 		    Tcl_DecrRefCount(subdirv[i]);
 		    subdirv[i] = copy;
 		    TclListObjLength(NULL, matchesObj, &end);
 		    while (repair < end) {
 			const char *bytes;
-			int numBytes;
+			Tcl_Size numBytes;
 			Tcl_Obj *fixme, *newObj;
 
 			Tcl_ListObjIndex(NULL, matchesObj, repair, &fixme);
@@ -2358,7 +2177,7 @@ DoGlob(
 				1, &newObj);
 			repair++;
 		    }
-		    repair = -1;
+		    repair = TCL_INDEX_NONE;
 		}
 	    }
 	}
@@ -2371,7 +2190,7 @@ DoGlob(
      */
 
     if (*p == '\0') {
-	int length;
+	Tcl_Size length;
 	Tcl_DString append;
 
 	/*
@@ -2390,7 +2209,7 @@ DoGlob(
 	Tcl_DStringAppend(&append, pattern, p-pattern);
 
 	if (pathPtr != NULL) {
-	    (void)TclGetStringFromObj(pathPtr, &length);
+	    (void) TclGetStringFromObj(pathPtr, &length);
 	} else {
 	    length = 0;
 	}
@@ -2424,7 +2243,7 @@ DoGlob(
 	 */
 
 	if (pathPtr == NULL) {
-	    joinedPtr = TclDStringToObj(&append);
+	    joinedPtr = Tcl_DStringToObj(&append);
 	} else if (flags) {
 	    joinedPtr = TclNewFSPathObj(pathPtr, Tcl_DStringValue(&append),
 		    Tcl_DStringLength(&append));
@@ -2435,8 +2254,8 @@ DoGlob(
 		 * The current prefix must end in a separator.
 		 */
 
-		int len;
-		const char *joined = TclGetStringFromObj(joinedPtr, &len);
+		Tcl_Size len;
+		const char *joined = TclGetStringFromObj(joinedPtr,&len);
 
 		if ((len > 0) && (strchr(separators, joined[len-1]) == NULL)) {
 		    Tcl_AppendToObj(joinedPtr, "/", 1);
@@ -2472,7 +2291,7 @@ DoGlob(
 	     * This behaviour is not currently tested for in the test suite.
 	     */
 
-	    int len;
+	    Tcl_Size len;
 	    const char *joined = TclGetStringFromObj(joinedPtr,&len);
 
 	    if ((len > 0) && (strchr(separators, joined[len-1]) == NULL)) {
@@ -2502,7 +2321,7 @@ DoGlob(
  *
  * Results:
  *	A pointer to a Tcl_StatBuf which may be deallocated by being passed to
- *	ckfree().
+ *	Tcl_Free().
  *
  * Side effects:
  *	None.
@@ -2513,7 +2332,7 @@ DoGlob(
 Tcl_StatBuf *
 Tcl_AllocStatBuf(void)
 {
-    return (Tcl_StatBuf *)ckalloc(sizeof(Tcl_StatBuf));
+    return (Tcl_StatBuf *)Tcl_Alloc(sizeof(Tcl_StatBuf));
 }
 
 /*
@@ -2522,8 +2341,8 @@ Tcl_AllocStatBuf(void)
  * Access functions for Tcl_StatBuf --
  *
  *	These functions provide portable read-only access to the portable
- *	fields of the Tcl_StatBuf structure (really a 'struct stat', 'struct
- *	stat64' or something else related). [TIP #316]
+ *	fields of the Tcl_StatBuf structure (really a 'struct stat'
+ *	or something else related). [TIP #316]
  *
  * Results:
  *	The value from the field being retrieved.
@@ -2538,21 +2357,21 @@ unsigned
 Tcl_GetFSDeviceFromStat(
     const Tcl_StatBuf *statPtr)
 {
-    return (unsigned)statPtr->st_dev;
+    return statPtr->st_dev;
 }
 
 unsigned
 Tcl_GetFSInodeFromStat(
     const Tcl_StatBuf *statPtr)
 {
-    return (unsigned)statPtr->st_ino;
+    return statPtr->st_ino;
 }
 
 unsigned
 Tcl_GetModeFromStat(
     const Tcl_StatBuf *statPtr)
 {
-    return (unsigned)statPtr->st_mode;
+    return statPtr->st_mode;
 }
 
 int
@@ -2566,7 +2385,7 @@ int
 Tcl_GetUserIdFromStat(
     const Tcl_StatBuf *statPtr)
 {
-    return (int)statPtr->st_uid;
+    return (int) statPtr->st_uid;
 }
 
 int
@@ -2583,61 +2402,66 @@ Tcl_GetDeviceTypeFromStat(
     return (int) statPtr->st_rdev;
 }
 
-Tcl_WideInt
+long long
 Tcl_GetAccessTimeFromStat(
     const Tcl_StatBuf *statPtr)
 {
-    return statPtr->st_atime;
+    return (long long) statPtr->st_atime;
 }
 
-Tcl_WideInt
+long long
 Tcl_GetModificationTimeFromStat(
     const Tcl_StatBuf *statPtr)
 {
-    return statPtr->st_mtime;
+    return (long long) statPtr->st_mtime;
 }
 
-Tcl_WideInt
+long long
 Tcl_GetChangeTimeFromStat(
     const Tcl_StatBuf *statPtr)
 {
-    return statPtr->st_ctime;
+    return (long long) statPtr->st_ctime;
 }
 
-Tcl_WideUInt
+unsigned long long
 Tcl_GetSizeFromStat(
     const Tcl_StatBuf *statPtr)
 {
-    return (Tcl_WideUInt)statPtr->st_size;
+    return (unsigned long long) statPtr->st_size;
 }
 
-Tcl_WideUInt
+unsigned long long
 Tcl_GetBlocksFromStat(
     const Tcl_StatBuf *statPtr)
 {
 #ifdef HAVE_STRUCT_STAT_ST_BLOCKS
-    return (Tcl_WideUInt)statPtr->st_blocks;
+    return (unsigned long long) statPtr->st_blocks;
 #else
     unsigned blksize = Tcl_GetBlockSizeFromStat(statPtr);
 
-    return ((Tcl_WideUInt)statPtr->st_size + blksize - 1) / blksize;
+    return ((unsigned long long) statPtr->st_size + blksize - 1) / blksize;
 #endif
 }
 
+#ifdef HAVE_STRUCT_STAT_ST_BLKSIZE
 unsigned
 Tcl_GetBlockSizeFromStat(
     const Tcl_StatBuf *statPtr)
 {
-#ifdef HAVE_STRUCT_STAT_ST_BLKSIZE
-    return (unsigned) statPtr->st_blksize;
+    return statPtr->st_blksize;
+}
 #else
+unsigned
+Tcl_GetBlockSizeFromStat(
+    TCL_UNUSED(const Tcl_StatBuf *))
+{
     /*
      * Not a great guess, but will do...
      */
 
     return GUESSED_BLOCK_SIZE;
-#endif
 }
+#endif
 
 /*
  * Local Variables:
