@@ -4,14 +4,15 @@
  *	This file implements functions that inspect CFBundle structures on
  *	MacOS X.
  *
- * Copyright 2001-2009, Apple Inc.
- * Copyright (c) 2003-2009 Daniel A. Steffen <das@users.sourceforge.net>
+ * Copyright © 2001-2009 Apple Inc.
+ * Copyright © 2003-2009 Daniel A. Steffen <das@users.sourceforge.net>
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
  */
 
 #include "tclPort.h"
+#include "tclInt.h"
 
 #ifdef HAVE_COREFOUNDATION
 #include <CoreFoundation/CoreFoundation.h>
@@ -106,7 +107,7 @@ OpenResourceMap(
 	if (tclMacOSXDarwinRelease >= 8)
 #endif
 	{
-	    openresourcemap = dlsym(RTLD_NEXT,
+	    openresourcemap = (short (*)(CFBundleRef))dlsym(RTLD_NEXT,
 		    "CFBundleOpenBundleResourceMap");
 #ifdef TCL_DEBUG_LOAD
 	    if (!openresourcemap) {
@@ -142,38 +143,6 @@ OpenResourceMap(
 
 #endif /* HAVE_COREFOUNDATION */
 
-/*
- *----------------------------------------------------------------------
- *
- * Tcl_MacOSXOpenBundleResources --
- *
- *	Given the bundle name for a shared library, this routine sets
- *	libraryPath to the Resources/Scripts directory in the framework
- *	package. If hasResourceFile is true, it will also open the main
- *	resource file for the bundle.
- *
- * Results:
- *	TCL_OK if the bundle could be opened, and the Scripts folder found.
- *	TCL_ERROR otherwise.
- *
- * Side effects:
- *	libraryVariableName may be set, and the resource file opened.
- *
- *----------------------------------------------------------------------
- */
-
-int
-Tcl_MacOSXOpenBundleResources(
-    Tcl_Interp *interp,
-    const char *bundleName,
-    int hasResourceFile,
-    int maxPathLen,
-    char *libraryPath)
-{
-    return Tcl_MacOSXOpenVersionedBundleResources(interp, bundleName, NULL,
-	    hasResourceFile, maxPathLen, libraryPath);
-}
-
 /*
  *----------------------------------------------------------------------
  *
@@ -197,11 +166,11 @@ Tcl_MacOSXOpenBundleResources(
 
 int
 Tcl_MacOSXOpenVersionedBundleResources(
-    Tcl_Interp *interp,
+    TCL_UNUSED(Tcl_Interp *),
     const char *bundleName,
     const char *bundleVersion,
     int hasResourceFile,
-    int maxPathLen,
+    Tcl_Size maxPathLen,
     char *libraryPath)
 {
 #ifdef HAVE_COREFOUNDATION
