@@ -5,7 +5,7 @@
  *	subcommands of the "file" command. All filename arguments should
  *	already be translated to native format.
  *
- * Copyright (c) 1996-1998 Sun Microsystems, Inc.
+ * Copyright © 1996-1998 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -13,7 +13,7 @@
  * Portions of this code were derived from NetBSD source code which has the
  * following copyright notice:
  *
- * Copyright (c) 1988, 1993, 1994
+ * Copyright © 1988, 1993, 1994
  *      The Regents of the University of California. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -41,8 +41,6 @@
  */
 
 #include "tclInt.h"
-#include <utime.h>
-#include <grp.h>
 #ifndef HAVE_STRUCT_STAT_ST_BLKSIZE
 #ifndef NO_FSTATFS
 #include <sys/statfs.h>
@@ -113,16 +111,8 @@ typedef int (TraversalProc)(Tcl_DString *srcPtr, Tcl_DString *dstPtr,
  * elsewhere in Tcl's core.
  */
 
-#ifdef DJGPP
+#ifndef DJGPP
 
-/*
- * See contrib/djgpp/tclDjgppFCmd.c for definition.
- */
-
-extern TclFileAttrProcs tclpFileAttrProcs[];
-extern const char *const tclpFileAttrStrings[];
-
-#else /* !DJGPP */
 enum {
 #if defined(__CYGWIN__)
     UNIX_ARCHIVE_ATTRIBUTE,
@@ -142,10 +132,9 @@ enum {
     MACOSX_CREATOR_ATTRIBUTE, MACOSX_TYPE_ATTRIBUTE, MACOSX_HIDDEN_ATTRIBUTE,
     MACOSX_RSRCLENGTH_ATTRIBUTE,
 #endif
-    UNIX_INVALID_ATTRIBUTE /* lint - last enum value needs no trailing , */
+    UNIX_INVALID_ATTRIBUTE
 };
 
-MODULE_SCOPE const char *const tclpFileAttrStrings[];
 const char *const tclpFileAttrStrings[] = {
 #if defined(__CYGWIN__)
     "-archive",
@@ -167,7 +156,6 @@ const char *const tclpFileAttrStrings[] = {
     NULL
 };
 
-MODULE_SCOPE const TclFileAttrProcs tclpFileAttrProcs[];
 const TclFileAttrProcs tclpFileAttrProcs[] = {
 #if defined(__CYGWIN__)
     {GetUnixFileAttributes, SetUnixFileAttributes},
@@ -256,7 +244,7 @@ Realpath(
 #endif /* PURIFY */
 
 #ifndef NO_REALPATH
-#if defined(__APPLE__) && defined(TCL_THREADS) && \
+#if defined(__APPLE__) && TCL_THREADS && \
 	defined(MAC_OS_X_VERSION_MIN_REQUIRED) && \
 	MAC_OS_X_VERSION_MIN_REQUIRED < 1030
 /*
@@ -270,13 +258,15 @@ MODULE_SCOPE long tclMacOSXDarwinRelease;
 #else
 #   define haveRealpath	1
 #endif
+#else /* NO_REALPATH */
+/*
+ * At least TclpObjNormalizedPath now requires REALPATH
+*/
+#error NO_REALPATH is not supported
 #endif /* NO_REALPATH */
 
 #ifdef HAVE_FTS
-#if defined(HAVE_STRUCT_STAT64) && !defined(__APPLE__)
-/* fts doesn't do stat64 */
-#   define noFtsStat	1
-#elif defined(__APPLE__) && defined(__LP64__) && \
+#if defined(__APPLE__) && defined(__LP64__) && \
 	defined(MAC_OS_X_VERSION_MIN_REQUIRED) && \
 	MAC_OS_X_VERSION_MIN_REQUIRED < 1050
 /*
@@ -331,8 +321,8 @@ TclpObjRenameFile(
     Tcl_Obj *srcPathPtr,
     Tcl_Obj *destPathPtr)
 {
-    return DoRenameFile(Tcl_FSGetNativePath(srcPathPtr),
-	    Tcl_FSGetNativePath(destPathPtr));
+    return DoRenameFile((const char *)Tcl_FSGetNativePath(srcPathPtr),
+	    (const char *)Tcl_FSGetNativePath(destPathPtr));
 }
 
 static int
@@ -447,14 +437,14 @@ TclpObjCopyFile(
     Tcl_Obj *srcPathPtr,
     Tcl_Obj *destPathPtr)
 {
-    const char *src = Tcl_FSGetNativePath(srcPathPtr);
+    const char *src = (const char *)Tcl_FSGetNativePath(srcPathPtr);
     Tcl_StatBuf srcStatBuf;
 
     if (TclOSlstat(src, &srcStatBuf) != 0) {		/* INTL: Native. */
 	return TCL_ERROR;
     }
 
-    return DoCopyFile(src, Tcl_FSGetNativePath(destPathPtr), &srcStatBuf);
+    return DoCopyFile(src, (const char *)Tcl_FSGetNativePath(destPathPtr), &srcStatBuf);
 }
 
 static int
@@ -494,8 +484,7 @@ DoCopyFile(
 	char linkBuf[MAXPATHLEN+1];
 	int length;
 
-	length = readlink(src, linkBuf, MAXPATHLEN);
-							/* INTL: Native. */
+	length = readlink(src, linkBuf, MAXPATHLEN);	/* INTL: Native. */
 	if (length == -1) {
 	    return TCL_ERROR;
 	}
@@ -554,9 +543,9 @@ TclUnixCopyFile(
     int dontCopyAtts)		/* If flag set, don't copy attributes. */
 {
     int srcFd, dstFd;
-    unsigned blockSize;		/* Optimal I/O blocksize for filesystem */
+    size_t blockSize;		/* Optimal I/O blocksize for filesystem */
     char *buffer;		/* Data buffer for copy */
-    size_t nread;
+    ssize_t nread;
 
 #ifdef DJGPP
 #define BINMODE |O_BINARY
@@ -610,21 +599,21 @@ TclUnixCopyFile(
     if (blockSize <= 0) {
 	blockSize = DEFAULT_COPY_BLOCK_SIZE;
     }
-    buffer = ckalloc(blockSize);
+    buffer = (char *)Tcl_Alloc(blockSize);
     while (1) {
-	nread = (size_t) read(srcFd, buffer, blockSize);
-	if ((nread == (size_t) -1) || (nread == 0)) {
+	nread = read(srcFd, buffer, blockSize);
+	if ((nread == -1) || (nread == 0)) {
 	    break;
 	}
-	if ((size_t) write(dstFd, buffer, nread) != nread) {
-	    nread = (size_t) -1;
+	if (write(dstFd, buffer, nread) != nread) {
+	    nread = -1;
 	    break;
 	}
     }
 
-    ckfree(buffer);
+    Tcl_Free(buffer);
     close(srcFd);
-    if ((close(dstFd) != 0) || (nread == (size_t) -1)) {
+    if ((close(dstFd) != 0) || (nread == -1)) {
 	unlink(dst);					/* INTL: Native. */
 	return TCL_ERROR;
     }
@@ -709,7 +698,7 @@ int
 TclpObjCreateDirectory(
     Tcl_Obj *pathPtr)
 {
-    return DoCreateDirectory(Tcl_FSGetNativePath(pathPtr));
+    return DoCreateDirectory((const char *)Tcl_FSGetNativePath(pathPtr));
 }
 
 static int
@@ -769,28 +758,35 @@ TclpObjCopyDirectory(
     Tcl_Obj *transPtr;
 
     transPtr = Tcl_FSGetTranslatedPath(NULL,srcPathPtr);
-    Tcl_UtfToExternalDString(NULL,
+    ret = Tcl_UtfToExternalDStringEx(NULL, NULL,
 	    (transPtr != NULL ? TclGetString(transPtr) : NULL),
-	    -1, &srcString);
+	    -1, 0, &srcString, NULL);
     if (transPtr != NULL) {
 	Tcl_DecrRefCount(transPtr);
     }
-    transPtr = Tcl_FSGetTranslatedPath(NULL,destPathPtr);
-    Tcl_UtfToExternalDString(NULL,
-	    (transPtr != NULL ? TclGetString(transPtr) : NULL),
-	    -1, &dstString);
-    if (transPtr != NULL) {
-	Tcl_DecrRefCount(transPtr);
-    }
-
-    ret = TraverseUnixTree(TraversalCopy, &srcString, &dstString, &ds, 0);
-
-    Tcl_DStringFree(&srcString);
-    Tcl_DStringFree(&dstString);
-
     if (ret != TCL_OK) {
-	*errorPtr = Tcl_NewStringObj(Tcl_DStringValue(&ds), -1);
-	Tcl_DStringFree(&ds);
+	*errorPtr = srcPathPtr;
+    } else {
+	transPtr = Tcl_FSGetTranslatedPath(NULL,destPathPtr);
+	ret = Tcl_UtfToExternalDStringEx(NULL, NULL,
+	    (transPtr != NULL ? TclGetString(transPtr) : NULL),
+	    -1, TCL_ENCODING_PROFILE_TCL8, &dstString, NULL);
+	if (transPtr != NULL) {
+	    Tcl_DecrRefCount(transPtr);
+	}
+	if (ret != TCL_OK) {
+	    *errorPtr = destPathPtr;
+	} else {
+	    ret = TraverseUnixTree(TraversalCopy, &srcString, &dstString, &ds, 0);
+	    /* Note above call only sets ds on error */
+	    if (ret != TCL_OK) {
+		*errorPtr = Tcl_DStringToObj(&ds);
+	    }
+	    Tcl_DStringFree(&dstString);
+	}
+	Tcl_DStringFree(&srcString);
+    }
+    if (ret != TCL_OK) {
 	Tcl_IncrRefCount(*errorPtr);
     }
     return ret;
@@ -813,7 +809,7 @@ TclpObjCopyDirectory(
  *	EEXIST:	    path is a non-empty directory.
  *	EINVAL:	    path is a root directory.
  *	ENOENT:	    path doesn't exist or is "".
- * 	ENOTDIR:    path is not a directory.
+ *	ENOTDIR:    path is not a directory.
  *
  * Side effects:
  *	Directory removed. If an error occurs, the error will be returned
@@ -833,18 +829,24 @@ TclpObjRemoveDirectory(
     int ret;
     Tcl_Obj *transPtr = Tcl_FSGetTranslatedPath(NULL, pathPtr);
 
-    Tcl_UtfToExternalDString(NULL,
+    ret = Tcl_UtfToExternalDStringEx(NULL, NULL,
 	    (transPtr != NULL ? TclGetString(transPtr) : NULL),
-	    -1, &pathString);
+	    -1, TCL_ENCODING_PROFILE_TCL8, &pathString, NULL);
     if (transPtr != NULL) {
 	Tcl_DecrRefCount(transPtr);
     }
-    ret = DoRemoveDirectory(&pathString, recursive, &ds);
-    Tcl_DStringFree(&pathString);
+    if (ret != TCL_OK) {
+	*errorPtr = pathPtr;
+    } else {
+	ret = DoRemoveDirectory(&pathString, recursive, &ds);
+	Tcl_DStringFree(&pathString);
+	/* Note above call only sets ds on error */
+	if (ret != TCL_OK) {
+	    *errorPtr = Tcl_DStringToObj(&ds);
+	}
+    }
 
     if (ret != TCL_OK) {
-	*errorPtr = Tcl_NewStringObj(Tcl_DStringValue(&ds), -1);
-	Tcl_DStringFree(&ds);
 	Tcl_IncrRefCount(*errorPtr);
     }
     return ret;
@@ -893,7 +895,7 @@ DoRemoveDirectory(
     result = TCL_OK;
     if ((errno != EEXIST) || (recursive == 0)) {
 	if (errorPtr != NULL) {
-	    Tcl_ExternalToUtfDString(NULL, path, -1, errorPtr);
+	    Tcl_ExternalToUtfDStringEx(NULL, NULL, path, TCL_INDEX_NONE, 0, errorPtr, NULL);
 	}
 	result = TCL_ERROR;
     }
@@ -951,17 +953,17 @@ TraverseUnixTree(
 				 * filled with UTF-8 name of file causing
 				 * error. */
     int doRewind)		/* Flag indicating that to ensure complete
-    				 * traversal of source hierarchy, the readdir
-    				 * loop should be rewound whenever
-    				 * traverseProc has returned TCL_OK; this is
-    				 * required when traverseProc modifies the
-    				 * source hierarchy, e.g. by deleting
-    				 * files. */
+				 * traversal of source hierarchy, the readdir
+				 * loop should be rewound whenever
+				 * traverseProc has returned TCL_OK; this is
+				 * required when traverseProc modifies the
+				 * source hierarchy, e.g. by deleting
+				 * files. */
 {
     Tcl_StatBuf statBuf;
     const char *source, *errfile;
-    int result, sourceLen;
-    int targetLen;
+    int result;
+    size_t targetLen, sourceLen;
 #ifndef HAVE_FTS
     int numProcessed = 0;
     Tcl_DirEntry *dirEntPtr;
@@ -974,7 +976,7 @@ TraverseUnixTree(
 
     errfile = NULL;
     result = TCL_OK;
-    targetLen = 0;		/* lint. */
+    targetLen = 0;
 
     source = Tcl_DStringValue(sourcePtr);
     if (TclOSlstat(source, &statBuf) != 0) {		/* INTL: Native. */
@@ -1025,9 +1027,9 @@ TraverseUnixTree(
 	 * Append name after slash, and recurse on the file.
 	 */
 
-	Tcl_DStringAppend(sourcePtr, dirEntPtr->d_name, -1);
+	Tcl_DStringAppend(sourcePtr, dirEntPtr->d_name, TCL_INDEX_NONE);
 	if (targetPtr != NULL) {
-	    Tcl_DStringAppend(targetPtr, dirEntPtr->d_name, -1);
+	    Tcl_DStringAppend(targetPtr, dirEntPtr->d_name, TCL_INDEX_NONE);
 	}
 	result = TraverseUnixTree(traverseProc, sourcePtr, targetPtr,
 		errorPtr, doRewind);
@@ -1142,7 +1144,7 @@ TraverseUnixTree(
   end:
     if (errfile != NULL) {
 	if (errorPtr != NULL) {
-	    Tcl_ExternalToUtfDString(NULL, errfile, -1, errorPtr);
+	    Tcl_ExternalToUtfDStringEx(NULL, NULL, errfile, TCL_INDEX_NONE, 0, errorPtr, NULL);
 	}
 	result = TCL_ERROR;
     }
@@ -1212,8 +1214,8 @@ TraversalCopy(
      */
 
     if (errorPtr != NULL) {
-	Tcl_ExternalToUtfDString(NULL, Tcl_DStringValue(dstPtr),
-		Tcl_DStringLength(dstPtr), errorPtr);
+	Tcl_ExternalToUtfDStringEx(NULL, NULL, Tcl_DStringValue(dstPtr),
+		Tcl_DStringLength(dstPtr), 0, errorPtr, NULL);
     }
     return TCL_ERROR;
 }
@@ -1240,14 +1242,14 @@ TraversalCopy(
 static int
 TraversalDelete(
     Tcl_DString *srcPtr,	/* Source pathname (native). */
-    Tcl_DString *ignore,	/* Destination pathname (not used). */
-    const Tcl_StatBuf *statBufPtr,
-				/* Stat info for file specified by srcPtr. */
+    TCL_UNUSED(Tcl_DString *),
+    TCL_UNUSED(const Tcl_StatBuf *),
     int type,			/* Reason for call - see TraverseUnixTree(). */
     Tcl_DString *errorPtr)	/* If non-NULL, uninitialized or free DString
 				 * filled with UTF-8 name of file causing
 				 * error. */
 {
+
     switch (type) {
     case DOTREE_F:
 	if (TclpDeleteFile(Tcl_DStringValue(srcPtr)) == 0) {
@@ -1263,8 +1265,8 @@ TraversalDelete(
 	break;
     }
     if (errorPtr != NULL) {
-	Tcl_ExternalToUtfDString(NULL, Tcl_DStringValue(srcPtr),
-		Tcl_DStringLength(srcPtr), errorPtr);
+	Tcl_ExternalToUtfDStringEx(NULL, NULL, Tcl_DStringValue(srcPtr),
+		Tcl_DStringLength(srcPtr), 0, errorPtr, NULL);
     }
     return TCL_ERROR;
 }
@@ -1289,7 +1291,11 @@ TraversalDelete(
 
 static int
 CopyFileAtts(
-    const char *src,		/* Path name of source file (native). */
+#ifdef MAC_OSX_TCL
+    const char *src,	/* Path name of source file (native). */
+#else
+    TCL_UNUSED(const char *) /*src*/,
+#endif
     const char *dst,		/* Path name of target file (native). */
     const Tcl_StatBuf *statBufPtr)
 				/* Stat info for source file */
@@ -1315,8 +1321,8 @@ CopyFileAtts(
 	}
     }
 
-    tval.actime = statBufPtr->st_atime;
-    tval.modtime = statBufPtr->st_mtime;
+    tval.actime = Tcl_GetAccessTimeFromStat(statBufPtr);
+    tval.modtime = Tcl_GetModificationTimeFromStat(statBufPtr);
 
     if (utime(dst, &tval)) {				/* INTL: Native. */
 	return TCL_ERROR;
@@ -1347,7 +1353,7 @@ CopyFileAtts(
 static int
 GetGroupAttribute(
     Tcl_Interp *interp,		/* The interp we are using for errors. */
-    int objIndex,		/* The index of the attribute. */
+    TCL_UNUSED(int) /*objIndex*/,
     Tcl_Obj *fileName,		/* The name of the file (UTF-8). */
     Tcl_Obj **attributePtrPtr)	/* A pointer to return the object with. */
 {
@@ -1369,13 +1375,13 @@ GetGroupAttribute(
     groupPtr = TclpGetGrGid(statBuf.st_gid);
 
     if (groupPtr == NULL) {
-	TclNewIntObj(*attributePtrPtr, (int) statBuf.st_gid);
+	TclNewIntObj(*attributePtrPtr, statBuf.st_gid);
     } else {
 	Tcl_DString ds;
 	const char *utf;
 
-	utf = Tcl_ExternalToUtfDString(NULL, groupPtr->gr_name, -1, &ds);
-	*attributePtrPtr = Tcl_NewStringObj(utf, -1);
+	utf = Tcl_ExternalToUtfDString(NULL, groupPtr->gr_name, TCL_INDEX_NONE, &ds);
+	*attributePtrPtr = Tcl_NewStringObj(utf, TCL_INDEX_NONE);
 	Tcl_DStringFree(&ds);
     }
     return TCL_OK;
@@ -1401,7 +1407,7 @@ GetGroupAttribute(
 static int
 GetOwnerAttribute(
     Tcl_Interp *interp,		/* The interp we are using for errors. */
-    int objIndex,		/* The index of the attribute. */
+    TCL_UNUSED(int) /*objIndex*/,
     Tcl_Obj *fileName,		/* The name of the file (UTF-8). */
     Tcl_Obj **attributePtrPtr)	/* A pointer to return the object with. */
 {
@@ -1423,12 +1429,12 @@ GetOwnerAttribute(
     pwPtr = TclpGetPwUid(statBuf.st_uid);
 
     if (pwPtr == NULL) {
-	TclNewIntObj(*attributePtrPtr, (int) statBuf.st_uid);
+	TclNewIntObj(*attributePtrPtr, statBuf.st_uid);
     } else {
 	Tcl_DString ds;
 
-	(void) Tcl_ExternalToUtfDString(NULL, pwPtr->pw_name, -1, &ds);
-	*attributePtrPtr = TclDStringToObj(&ds);
+	(void)Tcl_ExternalToUtfDString(NULL, pwPtr->pw_name, TCL_INDEX_NONE, &ds);
+	*attributePtrPtr = Tcl_DStringToObj(&ds);
     }
     return TCL_OK;
 }
@@ -1453,7 +1459,7 @@ GetOwnerAttribute(
 static int
 GetPermissionsAttribute(
     Tcl_Interp *interp,		    /* The interp we are using for errors. */
-    int objIndex,		    /* The index of the attribute. */
+    TCL_UNUSED(int) /*objIndex*/,
     Tcl_Obj *fileName,		    /* The name of the file (UTF-8). */
     Tcl_Obj **attributePtrPtr)	    /* A pointer to return the object with. */
 {
@@ -1472,7 +1478,7 @@ GetPermissionsAttribute(
     }
 
     *attributePtrPtr = Tcl_ObjPrintf(
-	    "%0#5lo", (long) (statBuf.st_mode & 0x00007FFF));
+	    "%0#5o", ((int)statBuf.st_mode & 0x7FFF));
     return TCL_OK;
 }
 
@@ -1495,23 +1501,27 @@ GetPermissionsAttribute(
 static int
 SetGroupAttribute(
     Tcl_Interp *interp,		/* The interp for error reporting. */
-    int objIndex,		/* The index of the attribute. */
+    TCL_UNUSED(int) /*objIndex*/,
     Tcl_Obj *fileName,		/* The name of the file (UTF-8). */
     Tcl_Obj *attributePtr)	/* New group for file. */
 {
-    long gid;
+    Tcl_WideInt gid;
     int result;
     const char *native;
 
-    if (Tcl_GetLongFromObj(NULL, attributePtr, &gid) != TCL_OK) {
+    if (TclGetWideIntFromObj(NULL, attributePtr, &gid) != TCL_OK) {
 	Tcl_DString ds;
 	struct group *groupPtr = NULL;
 	const char *string;
-	int length;
+	Tcl_Size length;
 
-	string = Tcl_GetStringFromObj(attributePtr, &length);
+	string = TclGetStringFromObj(attributePtr, &length);
 
-	native = Tcl_UtfToExternalDString(NULL, string, length, &ds);
+	if (Tcl_UtfToExternalDStringEx(interp, NULL, string, length, 0, &ds, NULL) != TCL_OK) {
+	    Tcl_DStringFree(&ds);
+	    return TCL_ERROR;
+	}
+	native = Tcl_DStringValue(&ds);
 	groupPtr = TclpGetGrNam(native); /* INTL: Native. */
 	Tcl_DStringFree(&ds);
 
@@ -1522,14 +1532,14 @@ SetGroupAttribute(
 			" group \"%s\" does not exist",
 			TclGetString(fileName), string));
 		Tcl_SetErrorCode(interp, "TCL", "OPERATION", "SETGRP",
-			"NO_GROUP", NULL);
+			"NO_GROUP", (char *)NULL);
 	    }
 	    return TCL_ERROR;
 	}
 	gid = groupPtr->gr_gid;
     }
 
-    native = Tcl_FSGetNativePath(fileName);
+    native = (const char *)Tcl_FSGetNativePath(fileName);
     result = chown(native, (uid_t) -1, (gid_t) gid);	/* INTL: Native. */
 
     if (result != 0) {
@@ -1562,23 +1572,27 @@ SetGroupAttribute(
 static int
 SetOwnerAttribute(
     Tcl_Interp *interp,		/* The interp for error reporting. */
-    int objIndex,		/* The index of the attribute. */
+    TCL_UNUSED(int) /*objIndex*/,
     Tcl_Obj *fileName,		/* The name of the file (UTF-8). */
     Tcl_Obj *attributePtr)	/* New owner for file. */
 {
-    long uid;
+    Tcl_WideInt uid;
     int result;
     const char *native;
 
-    if (Tcl_GetLongFromObj(NULL, attributePtr, &uid) != TCL_OK) {
+    if (TclGetWideIntFromObj(NULL, attributePtr, &uid) != TCL_OK) {
 	Tcl_DString ds;
 	struct passwd *pwPtr = NULL;
 	const char *string;
-	int length;
+	Tcl_Size length;
 
-	string = Tcl_GetStringFromObj(attributePtr, &length);
+	string = TclGetStringFromObj(attributePtr, &length);
 
-	native = Tcl_UtfToExternalDString(NULL, string, length, &ds);
+	if (Tcl_UtfToExternalDStringEx(interp, NULL, string, length, 0, &ds, NULL) != TCL_OK) {
+	    Tcl_DStringFree(&ds);
+	    return TCL_ERROR;
+	}
+	native = Tcl_DStringValue(&ds);
 	pwPtr = TclpGetPwNam(native);			/* INTL: Native. */
 	Tcl_DStringFree(&ds);
 
@@ -1589,14 +1603,14 @@ SetOwnerAttribute(
 			" user \"%s\" does not exist",
 			TclGetString(fileName), string));
 		Tcl_SetErrorCode(interp, "TCL", "OPERATION", "SETOWN",
-			"NO_USER", NULL);
+			"NO_USER", (char *)NULL);
 	    }
 	    return TCL_ERROR;
 	}
 	uid = pwPtr->pw_uid;
     }
 
-    native = Tcl_FSGetNativePath(fileName);
+    native = (const char *)Tcl_FSGetNativePath(fileName);
     result = chown(native, (uid_t) uid, (gid_t) -1);	/* INTL: Native. */
 
     if (result != 0) {
@@ -1629,11 +1643,11 @@ SetOwnerAttribute(
 static int
 SetPermissionsAttribute(
     Tcl_Interp *interp,		/* The interp we are using for errors. */
-    int objIndex,		/* The index of the attribute. */
+    TCL_UNUSED(int) /*objIndex*/,
     Tcl_Obj *fileName,		/* The name of the file (UTF-8). */
     Tcl_Obj *attributePtr)	/* The attribute to set. */
 {
-    long mode;
+    Tcl_WideInt mode;
     mode_t newMode;
     int result = TCL_ERROR;
     const char *native;
@@ -1651,12 +1665,12 @@ SetPermissionsAttribute(
 	Tcl_Obj *modeObj;
 
 	TclNewLiteralStringObj(modeObj, "0o");
-	Tcl_AppendToObj(modeObj, modeStringPtr+scanned+1, -1);
-	result = Tcl_GetLongFromObj(NULL, modeObj, &mode);
+	Tcl_AppendToObj(modeObj, modeStringPtr+scanned+1, TCL_INDEX_NONE);
+	result = TclGetWideIntFromObj(NULL, modeObj, &mode);
 	Tcl_DecrRefCount(modeObj);
     }
     if (result == TCL_OK
-	    || Tcl_GetLongFromObj(NULL, attributePtr, &mode) == TCL_OK) {
+	    || TclGetWideIntFromObj(NULL, attributePtr, &mode) == TCL_OK) {
 	newMode = (mode_t) (mode & 0x00007FFF);
     } else {
 	Tcl_StatBuf buf;
@@ -1684,13 +1698,13 @@ SetPermissionsAttribute(
 		Tcl_SetObjResult(interp, Tcl_ObjPrintf(
 			"unknown permission string format \"%s\"",
 			modeStringPtr));
-		Tcl_SetErrorCode(interp, "TCL", "VALUE", "PERMISSION", NULL);
+		Tcl_SetErrorCode(interp, "TCL", "VALUE", "PERMISSION", (char *)NULL);
 	    }
 	    return TCL_ERROR;
 	}
     }
 
-    native = Tcl_FSGetNativePath(fileName);
+    native = (const char *)Tcl_FSGetNativePath(fileName);
     result = chmod(native, newMode);		/* INTL: Native. */
     if (result != 0) {
 	if (interp != NULL) {
@@ -1751,7 +1765,7 @@ TclpObjListVolumes(void)
 
 static int
 GetModeFromPermString(
-    Tcl_Interp *interp,		/* The interp we are using for errors. */
+    TCL_UNUSED(Tcl_Interp *),
     const char *modeStringPtr, /* Permissions string */
     mode_t *modePtr)		/* pointer to the mode value */
 {
@@ -1771,7 +1785,7 @@ GetModeFromPermString(
 
     newMode = 0;
     for (i = 0; i < 9; i++) {
-	switch (*(modeStringPtr+i)) {
+	switch (modeStringPtr[i]) {
 	case 'r':
 	    if ((i%3) != 0) {
 		goto chmodStyleCheck;
@@ -1833,13 +1847,13 @@ GetModeFromPermString(
      * We now check for an "ugoa+-=rwxst" style permissions string
      */
 
-    for (n = 0 ; *(modeStringPtr+n) != '\0' ; n = n + i) {
+    for (n = 0 ; modeStringPtr[n] != '\0' ; n += i) {
 	oldMode = *modePtr;
 	who = op = what = op_found = who_found = 0;
-	for (i = 0 ; *(modeStringPtr+n+i) != '\0' ; i++ ) {
+	for (i = 0 ; modeStringPtr[n + i] != '\0' ; i++ ) {
 	    if (!who_found) {
 		/* who */
-		switch (*(modeStringPtr+n+i)) {
+		switch (modeStringPtr[n + i]) {
 		case 'u':
 		    who |= 0x9C0;
 		    continue;
@@ -1860,7 +1874,7 @@ GetModeFromPermString(
 	    }
 	    if (!op_found) {
 		/* op */
-		switch (*(modeStringPtr+n+i)) {
+		switch (modeStringPtr[n + i]) {
 		case '+':
 		    op = 1;
 		    op_found = 1;
@@ -1878,7 +1892,7 @@ GetModeFromPermString(
 		}
 	    }
 	    /* what */
-	    switch (*(modeStringPtr+n+i)) {
+	    switch (modeStringPtr[n + i]) {
 	    case 'r':
 		what |= 0x124;
 		continue;
@@ -1899,7 +1913,7 @@ GetModeFromPermString(
 	    default:
 		return TCL_ERROR;
 	    }
-	    if (*(modeStringPtr+n+i) == ',') {
+	    if (modeStringPtr[n + i] == ',') {
 		i++;
 		break;
 	    }
@@ -1924,17 +1938,14 @@ GetModeFromPermString(
  *
  * TclpObjNormalizePath --
  *
- *	This function scans through a path specification and replaces it, in
- *	place, with a normalized version. A normalized version is one in which
- *	all symlinks in the path are replaced with their expanded form (except
- *	a symlink at the very end of the path).
+ *	Replaces each component except that last one in a pathname that is a
+ *	symbolic link with the fully resolved target of that link.
  *
  * Results:
- *	The new 'nextCheckpoint' value, giving as far as we could understand
- *	in the path.
+ *	Stores the resulting path in pathPtr and returns the offset of the last
+ *	byte processed to obtain the resulting path.
  *
  * Side effects:
- *	The pathPtr string, is modified.
  *
  *---------------------------------------------------------------------------
  */
@@ -1942,46 +1953,55 @@ GetModeFromPermString(
 int
 TclpObjNormalizePath(
     Tcl_Interp *interp,
-    Tcl_Obj *pathPtr,
-    int nextCheckpoint)
+    Tcl_Obj *pathPtr,		/* An unshared object containing the path to
+				 * normalize. */
+    int nextCheckpoint)		/* offset to start at in pathPtr.  Must either
+				 * be 0 or the offset of a directory separator
+				 * at the end of a path part that is already
+				 * normalized.  I.e. this is not the index of
+				 * the byte just after the separator.  */
+
 {
     const char *currentPathEndPosition;
-    int pathLen;
     char cur;
-    const char *path = Tcl_GetStringFromObj(pathPtr, &pathLen);
+    Tcl_Size pathLen;
+    const char *path = TclGetStringFromObj(pathPtr, &pathLen);
     Tcl_DString ds;
     const char *nativePath;
 #ifndef NO_REALPATH
     char normPath[MAXPATHLEN];
 #endif
 
-    /*
-     * We add '1' here because if nextCheckpoint is zero we know that '/'
-     * exists, and if it isn't zero, it must point at a directory separator
-     * which we also know exists.
-     */
-
     currentPathEndPosition = path + nextCheckpoint;
     if (*currentPathEndPosition == '/') {
 	currentPathEndPosition++;
     }
 
 #ifndef NO_REALPATH
-    /*
-     * For speed, try to get the entire path in one go.
-     */
-
     if (nextCheckpoint == 0 && haveRealpath) {
-	char *lastDir = strrchr(currentPathEndPosition, '/');
+	/*
+	 * Try to get the entire path in one go
+	 */
+
+	const char *lastDir = strrchr(currentPathEndPosition, '/');
 
 	if (lastDir != NULL) {
-	    nativePath = Tcl_UtfToExternalDString(NULL, path,
-		    lastDir-path, &ds);
+	    if (Tcl_UtfToExternalDStringEx(interp, NULL, path,
+		    lastDir-path, 0, &ds, NULL) != TCL_OK) {
+		Tcl_DStringFree(&ds);
+		return -1;
+	    }
+	    nativePath = Tcl_DStringValue(&ds);
 	    if (Realpath(nativePath, normPath) != NULL) {
 		if (*nativePath != '/' && *normPath == '/') {
 		    /*
-		     * realpath has transformed a relative path into an
-		     * absolute path, we do not know how to handle this.
+		     * realpath transformed a relative path into an
+		     * absolute path.  Fall back to the long way.
+		     */
+
+		    /*
+		     * To do: This logic seems to be out of date.  This whole
+		     * routine should be reviewed and cleaed up.
 		     */
 		} else {
 		    nextCheckpoint = lastDir - path;
@@ -2006,8 +2026,12 @@ TclpObjNormalizePath(
 
 	    int accessOk;
 
-	    nativePath = Tcl_UtfToExternalDString(NULL, path,
-		    currentPathEndPosition - path, &ds);
+	    if (Tcl_UtfToExternalDStringEx(interp, NULL, path,
+		    currentPathEndPosition - path, 0, &ds, NULL) != TCL_OK) {
+		Tcl_DStringFree(&ds);
+		return -1;
+	    }
+	    nativePath = Tcl_DStringValue(&ds);
 	    accessOk = access(nativePath, F_OK);
 	    Tcl_DStringFree(&ds);
 
@@ -2020,13 +2044,13 @@ TclpObjNormalizePath(
 	    }
 
 	    /*
-	     * Update the acceptable point.
+	     * Assign the end of the current component to nextCheckpoint
 	     */
 
 	    nextCheckpoint = currentPathEndPosition - path;
 	} else if (cur == 0) {
 	    /*
-	     * Reached end of string.
+	     * The end of the string.
 	     */
 
 	    break;
@@ -2035,46 +2059,48 @@ TclpObjNormalizePath(
     }
 
     /*
-     * We should really now convert this to a canonical path. We do that with
-     * 'realpath' if we have it available. Otherwise we could step through
-     * every single path component, checking whether it is a symlink, but that
-     * would be a lot of work, and most modern OSes have 'realpath'.
+     * Call 'realpath' to obtain a canonical path.
      */
 
 #ifndef NO_REALPATH
     if (haveRealpath) {
-	/*
-	 * If we only had '/foo' or '/' then we never increment nextCheckpoint
-	 * and we don't need or want to go through 'Realpath'. Also, on some
-	 * platforms, passing an empty string to 'Realpath' will give us the
-	 * normalized pwd, which is not what we want at all!
-	 */
-
 	if (nextCheckpoint == 0) {
+	    /*
+	     * The path contains at most one component, e.g. '/foo' or '/',
+	     * so there is nothing to resolve. Also, on some platforms
+	     * 'Realpath' transforms an empty string into the normalized pwd,
+	     * which is the wrong answer.
+	     */
+
 	    return 0;
 	}
 
-	nativePath = Tcl_UtfToExternalDString(NULL, path,nextCheckpoint, &ds);
+	if (Tcl_UtfToExternalDStringEx(interp, NULL, path,nextCheckpoint, 0, &ds, NULL)) {
+	    Tcl_DStringFree(&ds);
+	    return -1;
+	}
+	nativePath = Tcl_DStringValue(&ds);
 	if (Realpath(nativePath, normPath) != NULL) {
-	    int newNormLen;
+	    Tcl_Size newNormLen;
 
 	wholeStringOk:
 	    newNormLen = strlen(normPath);
 	    if ((newNormLen == Tcl_DStringLength(&ds))
 		    && (strcmp(normPath, nativePath) == 0)) {
 		/*
-		 * String is unchanged.
+		 * The original path is unchanged.
 		 */
 
 		Tcl_DStringFree(&ds);
 
 		/*
-		 * Enable this to have the native FS claim normalization of
-		 * the whole path for existing files. That would permit the
-		 * caller to declare normalization complete without calls to
-		 * additional filesystems. Saving lots of calls is probably
-		 * worth the extra access() time here. When no other FS's are
-		 * registered though, things are less clear.
+		 * Uncommenting this would mean that this native filesystem
+		 * routine claims the path is normalized if the file exists,
+		 * which would permit the caller to avoid iterating through
+		 * other filesystems. Saving lots of calls is
+		 * probably worth the extra access() time, but in the common
+		 * case that no other filesystems are registered this is an
+		 * unnecessary expense.
 		 *
 		if (0 == access(normPath, F_OK)) {
 		    return pathLen;
@@ -2085,16 +2111,15 @@ TclpObjNormalizePath(
 	    }
 
 	    /*
-	     * Free up the native path and put in its place the converted,
-	     * normalized path.
+	     * Free the original path and replace it with the normalized path.
 	     */
 
 	    Tcl_DStringFree(&ds);
-	    Tcl_ExternalToUtfDString(NULL, normPath, (int) newNormLen, &ds);
+	    Tcl_ExternalToUtfDStringEx(NULL, NULL, normPath, newNormLen, 0, &ds, NULL);
 
 	    if (path[nextCheckpoint] != '\0') {
 		/*
-		 * Not at end, append remaining path.
+		 * Append the remaining path components.
 		 */
 
 		int normLen = Tcl_DStringLength(&ds);
@@ -2103,7 +2128,8 @@ TclpObjNormalizePath(
 			pathLen - nextCheckpoint);
 
 		/*
-		 * We recognise up to and including the directory separator.
+		 * characters up to and including the directory separator have
+		 * been processed
 		 */
 
 		nextCheckpoint = normLen + 1;
@@ -2115,10 +2141,6 @@ TclpObjNormalizePath(
 		nextCheckpoint = Tcl_DStringLength(&ds);
 	    }
 
-	    /*
-	     * Overwrite with the normalized path.
-	     */
-
 	    Tcl_SetStringObj(pathPtr, Tcl_DStringValue(&ds),
 		    Tcl_DStringLength(&ds));
 	}
@@ -2175,56 +2197,68 @@ TclUnixOpenTemporaryFile(
     Tcl_Obj *extensionObj,
     Tcl_Obj *resultingNameObj)
 {
-    Tcl_DString template, tmp;
+    Tcl_DString templ, tmp;
     const char *string;
-    int len, fd;
+    int fd;
+    Tcl_Size length;
 
     /*
-     * We should also check against making more then TMP_MAX of these.
+     * We should also check against making more than TMP_MAX of these.
      */
 
     if (dirObj) {
-	string = Tcl_GetStringFromObj(dirObj, &len);
-	Tcl_UtfToExternalDString(NULL, string, len, &template);
+	string = TclGetStringFromObj(dirObj, &length);
+	if (Tcl_UtfToExternalDStringEx(NULL, NULL, string, length, 0, &templ, NULL) != TCL_OK) {
+	    return -1;
+	}
     } else {
-	Tcl_DStringInit(&template);
-	Tcl_DStringAppend(&template, DefaultTempDir(), -1); /* INTL: native */
+	Tcl_DStringInit(&templ);
+	Tcl_DStringAppend(&templ, DefaultTempDir(), TCL_INDEX_NONE); /* INTL: native */
     }
 
-    TclDStringAppendLiteral(&template, "/");
+    TclDStringAppendLiteral(&templ, "/");
 
     if (basenameObj) {
-	string = Tcl_GetStringFromObj(basenameObj, &len);
-	Tcl_UtfToExternalDString(NULL, string, len, &tmp);
-	TclDStringAppendDString(&template, &tmp);
+	string = TclGetStringFromObj(basenameObj, &length);
+	if (Tcl_UtfToExternalDStringEx(NULL, NULL, string, length, 0, &tmp, NULL) != TCL_OK) {
+	    Tcl_DStringFree(&tmp);
+	    return -1;
+	}
+	TclDStringAppendDString(&templ, &tmp);
 	Tcl_DStringFree(&tmp);
     } else {
-	TclDStringAppendLiteral(&template, "tcl");
+	TclDStringAppendLiteral(&templ, "tcl");
     }
 
-    TclDStringAppendLiteral(&template, "_XXXXXX");
+    TclDStringAppendLiteral(&templ, "_XXXXXX");
 
 #ifdef HAVE_MKSTEMPS
     if (extensionObj) {
-	string = Tcl_GetStringFromObj(extensionObj, &len);
-	Tcl_UtfToExternalDString(NULL, string, len, &tmp);
-	TclDStringAppendDString(&template, &tmp);
-	fd = mkstemps(Tcl_DStringValue(&template), Tcl_DStringLength(&tmp));
+	string = TclGetStringFromObj(extensionObj, &length);
+	if (Tcl_UtfToExternalDStringEx(NULL, NULL, string, length, 0, &tmp, NULL) != TCL_OK) {
+	    Tcl_DStringFree(&templ);
+	    return -1;
+	}
+	TclDStringAppendDString(&templ, &tmp);
+	fd = mkstemps(Tcl_DStringValue(&templ), Tcl_DStringLength(&tmp));
 	Tcl_DStringFree(&tmp);
     } else
 #endif
     {
-	fd = mkstemp(Tcl_DStringValue(&template));
+	fd = mkstemp(Tcl_DStringValue(&templ));
     }
 
     if (fd == -1) {
-	Tcl_DStringFree(&template);
+	Tcl_DStringFree(&templ);
 	return -1;
     }
 
     if (resultingNameObj) {
-	Tcl_ExternalToUtfDString(NULL, Tcl_DStringValue(&template),
-		Tcl_DStringLength(&template), &tmp);
+	if (Tcl_ExternalToUtfDStringEx(NULL, NULL, Tcl_DStringValue(&templ),
+		Tcl_DStringLength(&templ), 0, &tmp, NULL) != TCL_OK) {
+	    Tcl_DStringFree(&templ);
+	    return -1;
+	}
 	Tcl_SetStringObj(resultingNameObj, Tcl_DStringValue(&tmp),
 		Tcl_DStringLength(&tmp));
 	Tcl_DStringFree(&tmp);
@@ -2235,10 +2269,10 @@ TclUnixOpenTemporaryFile(
 	 * this!
 	 */
 
-	unlink(Tcl_DStringValue(&template));
+	unlink(Tcl_DStringValue(&templ));
 	errno = 0;
     }
-    Tcl_DStringFree(&template);
+    Tcl_DStringFree(&templ);
 
     return fd;
 }
@@ -2251,17 +2285,17 @@ static const char *
 DefaultTempDir(void)
 {
     const char *dir;
-    struct stat buf;
+    Tcl_StatBuf buf;
 
     dir = getenv("TMPDIR");
-    if (dir && dir[0] && stat(dir, &buf) == 0 && S_ISDIR(buf.st_mode)
+    if (dir && dir[0] && TclOSstat(dir, &buf) == 0 && S_ISDIR(buf.st_mode)
 	    && access(dir, W_OK) == 0) {
 	return dir;
     }
 
 #ifdef P_tmpdir
     dir = P_tmpdir;
-    if (stat(dir, &buf)==0 && S_ISDIR(buf.st_mode) && access(dir, W_OK)==0) {
+    if (TclOSstat(dir, &buf)==0 && S_ISDIR(buf.st_mode) && access(dir, W_OK)==0) {
 	return dir;
     }
 #endif
@@ -2275,6 +2309,93 @@ DefaultTempDir(void)
     return TCL_TEMPORARY_FILE_DIRECTORY;
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * TclpCreateTemporaryDirectory --
+ *
+ *	Creates a temporary directory, possibly based on the supplied bits and
+ *	pieces of template supplied in the arguments.
+ *
+ * Results:
+ *	An object (refcount 0) containing the name of the newly-created
+ *	directory, or NULL on failure.
+ *
+ * Side effects:
+ *	Accesses the native filesystem. Makes a directory.
+ *
+ *----------------------------------------------------------------------
+ */
+
+Tcl_Obj *
+TclpCreateTemporaryDirectory(
+    Tcl_Obj *dirObj,
+    Tcl_Obj *basenameObj)
+{
+    Tcl_DString templ, tmp;
+    const char *string;
+
+#define DEFAULT_TEMP_DIR_PREFIX	"tcl"
+
+    /*
+     * Build the template in writable memory from the user-supplied pieces and
+     * some defaults.
+     */
+
+    if (dirObj) {
+	string = TclGetString(dirObj);
+	if (Tcl_UtfToExternalDStringEx(NULL, NULL, string, dirObj->length, 0, &templ, NULL) != TCL_OK) {
+	    return NULL;
+	}
+    } else {
+	Tcl_DStringInit(&templ);
+	Tcl_DStringAppend(&templ, DefaultTempDir(), TCL_INDEX_NONE); /* INTL: native */
+    }
+
+    if (Tcl_DStringValue(&templ)[Tcl_DStringLength(&templ) - 1] != '/') {
+	TclDStringAppendLiteral(&templ, "/");
+    }
+
+    if (basenameObj) {
+	string = TclGetString(basenameObj);
+	if (basenameObj->length) {
+	    if (Tcl_UtfToExternalDStringEx(NULL, NULL, string, basenameObj->length, 0, &tmp, NULL) != TCL_OK) {
+		Tcl_DStringFree(&templ);
+		return NULL;
+	    }
+	    TclDStringAppendDString(&templ, &tmp);
+	    Tcl_DStringFree(&tmp);
+	} else {
+	    TclDStringAppendLiteral(&templ, DEFAULT_TEMP_DIR_PREFIX);
+	}
+    } else {
+	TclDStringAppendLiteral(&templ, DEFAULT_TEMP_DIR_PREFIX);
+    }
+
+    TclDStringAppendLiteral(&templ, "_XXXXXX");
+
+    /*
+     * Make the temporary directory.
+     */
+
+    if (mkdtemp(Tcl_DStringValue(&templ)) == NULL) {
+	Tcl_DStringFree(&templ);
+	return NULL;
+    }
+
+    /*
+     * The template has been updated. Tell the caller what it was.
+     */
+
+    if (Tcl_ExternalToUtfDStringEx(NULL, NULL, Tcl_DStringValue(&templ),
+	    Tcl_DStringLength(&templ), 0, &tmp, NULL) != TCL_OK) {
+	Tcl_DStringFree(&templ);
+	return NULL;
+    }
+    Tcl_DStringFree(&templ);
+    return Tcl_DStringToObj(&tmp);
+}
+
 #if defined(__CYGWIN__)
 
 static void
@@ -2283,7 +2404,7 @@ StatError(
     Tcl_Obj *fileName)		/* The name of the file which caused the
 				 * error. */
 {
-    TclWinConvertError(GetLastError());
+    Tcl_WinConvertError(GetLastError());
     Tcl_SetObjResult(interp, Tcl_ObjPrintf("could not read \"%s\": %s",
 	    TclGetString(fileName), Tcl_PosixError(interp)));
 }
@@ -2292,57 +2413,60 @@ static WCHAR *
 winPathFromObj(
     Tcl_Obj *fileName)
 {
-    int size;
-    const char *native =  Tcl_FSGetNativePath(fileName);
+    size_t size;
+    const char *native =  (const char *)Tcl_FSGetNativePath(fileName);
     WCHAR *winPath;
 
     size = cygwin_conv_path(1, native, NULL, 0);
-    winPath = ckalloc(size);
+    winPath = (WCHAR *)Tcl_Alloc(size);
     cygwin_conv_path(1, native, winPath, size);
 
     return winPath;
 }
 
 static const int attributeArray[] = {
-    0x20, 0, 2, 0, 0, 1, 4};
+    0x20, 0, 2, 0, 0, 1, 4
+};
 
 /*
  *----------------------------------------------------------------------
  *
  * GetUnixFileAttributes
  *
- *	Gets the readonly attribute of a file.
+ *	Gets an attribute of a file.
  *
  * Results:
- *	Standard TCL result. Returns a new Tcl_Obj in attributePtrPtr if there
- *	is no error. The object will have ref count 0.
+ *	A standard Tcl result.
  *
  * Side effects:
- *	A new object is allocated.
+ *	If there is no error assigns to *attributePtrPtr the address of a new
+ *	Tcl_Obj having a refCount of zero and containing the value of the
+ *	specified attribute.
+ *
  *
  *----------------------------------------------------------------------
  */
 
 static int
 GetUnixFileAttributes(
-    Tcl_Interp *interp,		/* The interp we are using for errors. */
+    Tcl_Interp *interp,		/* The interp to report errors to. */
     int objIndex,		/* The index of the attribute. */
-    Tcl_Obj *fileName,		/* The name of the file (UTF-8). */
-    Tcl_Obj **attributePtrPtr)	/* A pointer to return the object with. */
+    Tcl_Obj *fileName,		/* The pathname of the file (UTF-8). */
+    Tcl_Obj **attributePtrPtr)	/* Where to store the result. */
 {
     int fileAttributes;
     WCHAR *winPath = winPathFromObj(fileName);
 
     fileAttributes = GetFileAttributesW(winPath);
-    ckfree(winPath);
+    Tcl_Free(winPath);
 
     if (fileAttributes == -1) {
 	StatError(interp, fileName);
 	return TCL_ERROR;
     }
 
-    TclNewIntObj(*attributePtrPtr, (fileAttributes&attributeArray[objIndex])!=0);
-
+    TclNewIntObj(*attributePtrPtr,
+	    (fileAttributes & attributeArray[objIndex]) != 0);
     return TCL_OK;
 }
 
@@ -2381,7 +2505,7 @@ SetUnixFileAttributes(
     fileAttributes = old = GetFileAttributesW(winPath);
 
     if (fileAttributes == -1) {
-	ckfree(winPath);
+	Tcl_Free(winPath);
 	StatError(interp, fileName);
 	return TCL_ERROR;
     }
@@ -2394,12 +2518,12 @@ SetUnixFileAttributes(
 
     if ((fileAttributes != old)
 	    && !SetFileAttributesW(winPath, fileAttributes)) {
-	ckfree(winPath);
+	Tcl_Free(winPath);
 	StatError(interp, fileName);
 	return TCL_ERROR;
     }
 
-	ckfree(winPath);
+    Tcl_Free(winPath);
     return TCL_OK;
 }
 #elif defined(HAVE_CHFLAGS) && defined(UF_IMMUTABLE)
@@ -2423,7 +2547,7 @@ SetUnixFileAttributes(
 static int
 GetUnixFileAttributes(
     Tcl_Interp *interp,		/* The interp we are using for errors. */
-    int objIndex,		/* The index of the attribute. */
+    TCL_UNUSED(int) /*objIndex*/,
     Tcl_Obj *fileName,		/* The name of the file (UTF-8). */
     Tcl_Obj **attributePtrPtr)	/* A pointer to return the object with. */
 {
@@ -2441,8 +2565,7 @@ GetUnixFileAttributes(
 	return TCL_ERROR;
     }
 
-    *attributePtrPtr = Tcl_NewBooleanObj(statBuf.st_flags&UF_IMMUTABLE);
-
+    TclNewIntObj(*attributePtrPtr, (statBuf.st_flags & UF_IMMUTABLE) != 0);
     return TCL_OK;
 }
 
@@ -2465,7 +2588,7 @@ GetUnixFileAttributes(
 static int
 SetUnixFileAttributes(
     Tcl_Interp *interp,		/* The interp we are using for errors. */
-    int objIndex,		/* The index of the attribute. */
+    TCL_UNUSED(int) /*objIndex*/,
     Tcl_Obj *fileName,		/* The name of the file (UTF-8). */
     Tcl_Obj *attributePtr)	/* The attribute to set. */
 {
@@ -2494,7 +2617,7 @@ SetUnixFileAttributes(
 	statBuf.st_flags &= ~UF_IMMUTABLE;
     }
 
-    native = Tcl_FSGetNativePath(fileName);
+    native = (const char *)Tcl_FSGetNativePath(fileName);
     result = chflags(native, statBuf.st_flags);		/* INTL: Native. */
     if (result != 0) {
 	if (interp != NULL) {
