@@ -7,7 +7,7 @@
  *	file that contains #ifdefs to handle different flavors of UNIX. This
  *	file sets up the union of all UNIX-related things needed by any of the
  *	Tcl core files. This file depends on configuration #defines such as
- *	NO_DIRENT_H that are set up by the "configure" script.
+ *	HAVE_SYS_PARAM_H that are set up by the "configure" script.
  *
  *	Much of the material in this file was originally contributed by Karl
  *	Lehenbauer, Mark Diekhans and Peter da Silva.
@@ -40,15 +40,7 @@
 #   include <sys/param.h>
 #endif
 #include <sys/types.h>
-#ifdef USE_DIRENT2_H
-#   include "../compat/dirent2.h"
-#else
-#ifdef NO_DIRENT_H
-#   include "../compat/dirent.h"
-#else
-#   include <dirent.h>
-#endif
-#endif
+#include <dirent.h>
 
 /*
  *---------------------------------------------------------------------------
@@ -90,11 +82,10 @@ typedef off_t		Tcl_SeekOffset;
 extern "C" {
 #endif
     /* Make some symbols available without including <windows.h> */
-#   define DWORD unsigned int
 #   define CP_UTF8 65001
 #   define GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS 0x00000004
-#   define HANDLE void *
-#   define HINSTANCE void *
+#   define HMODULE void *
+#   define MAX_PATH 260
 #   define SOCKET unsigned int
 #   define WSAEWOULDBLOCK 10035
     typedef unsigned short WCHAR;
@@ -118,15 +109,14 @@ extern "C" {
 #pragma clang diagnostic pop
 #endif
 #   define timezone _timezone
+    extern int TclOSfstat(int fd, void *statBuf);
     extern int TclOSstat(const char *name, void *statBuf);
     extern int TclOSlstat(const char *name, void *statBuf);
 #ifdef __cplusplus
 }
 #endif
-#elif defined(HAVE_STRUCT_STAT64) && !defined(__APPLE__)
-#   define TclOSstat(name, buf) stat64(name, (struct stat64 *)buf)
-#   define TclOSlstat(name,buf) lstat64(name, (struct stat64 *)buf)
 #else
+#   define TclOSfstat(fd, buf) fstat(fd, (struct stat *)buf)
 #   define TclOSstat(name, buf) stat(name, (struct stat *)buf)
 #   define TclOSlstat(name, buf) lstat(name, (struct stat *)buf)
 #endif
@@ -142,16 +132,10 @@ extern "C" {
 #   include <sys/select.h>
 #endif
 #include <sys/stat.h>
-#ifdef TIME_WITH_SYS_TIME
-#   include <sys/time.h>
-#   include <time.h>
-#else
 #ifdef HAVE_SYS_TIME_H
 #   include <sys/time.h>
-#else
-#   include <time.h>
-#endif
 #endif
+#include <time.h>
 #ifndef NO_SYS_WAIT_H
 #   include <sys/wait.h>
 #endif
@@ -159,16 +143,9 @@ extern "C" {
 #   include <inttypes.h>
 #endif
 #include <limits.h>
-#ifdef HAVE_STDINT_H
-#   include <stdint.h>
-#endif
-#ifdef HAVE_UNISTD_H
-#   include <unistd.h>
-#else
-#   include "../compat/unistd.h"
-#endif
+#include <unistd.h>
 
-extern int TclUnixSetBlockingMode(int fd, int mode);
+MODULE_SCOPE int TclUnixSetBlockingMode(int fd, int mode);
 
 #include <utime.h>
 
@@ -198,13 +175,7 @@ extern int TclUnixSetBlockingMode(int fd, int mode);
  *---------------------------------------------------------------------------
  */
 
-#ifndef NO_FLOAT_H
-#   include <float.h>
-#else
-#ifndef NO_VALUES_H
-#   include <values.h>
-#endif
-#endif
+#include <float.h>
 
 #ifndef FLT_MAX
 #   ifdef MAXFLOAT
@@ -625,10 +596,8 @@ extern char **		environ;
 #	    undef HAVE_COPYFILE
 #	endif
 #	if MAC_OS_X_VERSION_MAX_ALLOWED < 1030
-#	    ifdef TCL_THREADS
-		/* prior to 10.3, realpath is not threadsafe, c.f. bug 711232 */
-#		define NO_REALPATH 1
-#	    endif
+	    /* prior to 10.3, realpath is not threadsafe, c.f. bug 711232 */
+#	    define NO_REALPATH 1
 #	    undef HAVE_LANGINFO
 #	endif
 #   endif /* MAC_OS_X_VERSION_MAX_ALLOWED */
@@ -679,9 +648,9 @@ typedef int socklen_t;
  *---------------------------------------------------------------------------
  */
 
-#define TclpSysAlloc(size, isBin)	malloc((size_t)(size))
-#define TclpSysFree(ptr)		free((char *)(ptr))
-#define TclpSysRealloc(ptr, size)	realloc((char *)(ptr), (size_t)(size))
+#define TclpSysAlloc(size)		malloc(size)
+#define TclpSysFree(ptr)		free(ptr)
+#define TclpSysRealloc(ptr, size)	realloc(ptr, size)
 
 /*
  *---------------------------------------------------------------------------
@@ -689,9 +658,7 @@ typedef int socklen_t;
  *---------------------------------------------------------------------------
  */
 
-#define TclpExit	exit
-
-#ifdef TCL_THREADS
+#if !defined(TCL_THREADS) || TCL_THREADS
 #   include <pthread.h>
 #endif /* TCL_THREADS */
 
@@ -711,14 +678,14 @@ typedef int socklen_t;
 #include <pwd.h>
 #include <grp.h>
 
-extern struct passwd *	TclpGetPwNam(const char *name);
-extern struct group *	TclpGetGrNam(const char *name);
-extern struct passwd *	TclpGetPwUid(uid_t uid);
-extern struct group *	TclpGetGrGid(gid_t gid);
-extern struct hostent *	TclpGetHostByName(const char *name);
-extern struct hostent *	TclpGetHostByAddr(const char *addr,
+MODULE_SCOPE struct passwd *	TclpGetPwNam(const char *name);
+MODULE_SCOPE struct group *	TclpGetGrNam(const char *name);
+MODULE_SCOPE struct passwd *	TclpGetPwUid(uid_t uid);
+MODULE_SCOPE struct group *	TclpGetGrGid(gid_t gid);
+MODULE_SCOPE struct hostent *	TclpGetHostByName(const char *name);
+MODULE_SCOPE struct hostent *	TclpGetHostByAddr(const char *addr,
 				    int length, int type);
-extern void *TclpMakeTcpClientChannelMode(
+MODULE_SCOPE void *TclpMakeTcpClientChannelMode(
 				    void *tcpSocket, int mode);
 
 #endif /* _TCLUNIXPORT */
