@@ -52,8 +52,9 @@
  * The following flag is used in OpenKeys to indicate that the specified key
  * should be created if it doesn't currently exist.
  */
-
-#define REG_CREATE 1
+enum OpenKeysFlags {
+    REG_CREATE = 1
+};
 
 /*
  * The following tables contain the mapping from registry root names to the
@@ -86,12 +87,28 @@ static const char *const typeNames[] = {
 
 static DWORD lastType = REG_RESOURCE_LIST;
 
+#if (TCL_MAJOR_VERSION < 9) && defined(TCL_MINOR_VERSION) && (TCL_MINOR_VERSION < 7)
+# if TCL_UTF_MAX > 3
+#   define Tcl_WCharToUtfDString(a,b,c) Tcl_WinTCharToUtf((TCHAR *)(a),(b)*sizeof(WCHAR),c)
+#   define Tcl_UtfToWCharDString(a,b,c) (WCHAR *)Tcl_WinUtfToTChar(a,b,c)
+# else
+#   define Tcl_WCharToUtfDString Tcl_UniCharToUtfDString
+#   define Tcl_UtfToWCharDString Tcl_UtfToUniCharDString
+# endif
+#ifndef Tcl_Size
+#   define Tcl_Size int
+#endif
+#ifndef Tcl_CreateObjCommand2
+#   define Tcl_CreateObjCommand2 Tcl_CreateObjCommand
+#endif
+#endif
+
 /*
  * Declarations for functions defined in this file.
  */
 
 static void		AppendSystemError(Tcl_Interp *interp, DWORD error);
-static int		BroadcastValue(Tcl_Interp *interp, int objc,
+static int		BroadcastValue(Tcl_Interp *interp, Tcl_Size objc,
 			    Tcl_Obj *const objv[]);
 static DWORD		ConvertDWORD(DWORD type, DWORD value);
 static void		DeleteCmd(void *clientData);
@@ -118,46 +135,22 @@ static int		ParseKeyName(Tcl_Interp *interp, char *name,
 static DWORD		RecursiveDeleteKey(HKEY hStartKey,
 			    const WCHAR * pKeyName, REGSAM mode);
 static int		RegistryObjCmd(void *clientData,
-			    Tcl_Interp *interp, int objc,
+			    Tcl_Interp *interp, Tcl_Size objc,
 			    Tcl_Obj *const objv[]);
 static int		SetValue(Tcl_Interp *interp, Tcl_Obj *keyNameObj,
 			    Tcl_Obj *valueNameObj, Tcl_Obj *dataObj,
 			    Tcl_Obj *typeObj, REGSAM mode);
 
-#if (TCL_MAJOR_VERSION < 9) && (TCL_MINOR_VERSION < 7)
-# if TCL_UTF_MAX > 3
-#   define Tcl_WCharToUtfDString(a,b,c) Tcl_WinTCharToUtf((TCHAR *)(a),(b)*sizeof(WCHAR),c)
-#   define Tcl_UtfToWCharDString(a,b,c) (WCHAR *)Tcl_WinUtfToTChar(a,b,c)
-# else
-#   define Tcl_WCharToUtfDString Tcl_UniCharToUtfDString
-#   define Tcl_UtfToWCharDString Tcl_UtfToUniCharDString
-# endif
-#endif
-
-static unsigned char *
-getByteArrayFromObj(
-	Tcl_Obj *objPtr,
-	size_t *lengthPtr
-) {
-    int length;
-
-    unsigned char *result = Tcl_GetByteArrayFromObj(objPtr, &length);
-#if TCL_MAJOR_VERSION > 8
-    if (sizeof(TCL_HASH_TYPE) > sizeof(int)) {
-	/* 64-bit and TIP #494 situation: */
-	 *lengthPtr = *(TCL_HASH_TYPE *) objPtr->internalRep.twoPtrValue.ptr1;
-    } else
-#endif
-	/* 32-bit or without TIP #494 */
-    *lengthPtr = (size_t) (unsigned) length;
-    return result;
-}
-
 #ifdef __cplusplus
 extern "C" {
 #endif
 DLLEXPORT int		Registry_Init(Tcl_Interp *interp);
 DLLEXPORT int		Registry_Unload(Tcl_Interp *interp, int flags);
+#if TCL_MAJOR_VERSION < 9
+/* With those additional entries, "load tclregistry13.dll" works without 3th argument */
+DLLEXPORT int		Tclregistry_Init(Tcl_Interp *interp);
+DLLEXPORT int		Tclregistry_Unload(Tcl_Interp *interp, int flags);
+#endif
 #ifdef __cplusplus
 }
 #endif
@@ -184,15 +177,23 @@ Registry_Init(
 {
     Tcl_Command cmd;
 
-    if (Tcl_InitStubs(interp, "8.5", 0) == NULL) {
+    if (Tcl_InitStubs(interp, "8.5-", 0) == NULL) {
 	return TCL_ERROR;
     }
 
-    cmd = Tcl_CreateObjCommand(interp, "registry", RegistryObjCmd,
+    cmd = Tcl_CreateObjCommand2(interp, "registry", RegistryObjCmd,
 	    interp, DeleteCmd);
     Tcl_SetAssocData(interp, REGISTRY_ASSOC_KEY, NULL, cmd);
-    return Tcl_PkgProvideEx(interp, "registry", "1.3.5", NULL);
+    return Tcl_PkgProvideEx(interp, "registry", "1.3.7", NULL);
 }
+#if TCL_MAJOR_VERSION < 9
+int
+Tclregistry_Init(
+    Tcl_Interp *interp)
+{
+    return Registry_Init(interp);
+}
+#endif
 
 /*
  *----------------------------------------------------------------------
@@ -239,6 +240,15 @@ Registry_Unload(
 
     return TCL_OK;
 }
+#if TCL_MAJOR_VERSION < 9
+int
+Tclregistry_Unload(
+    Tcl_Interp *interp,
+    int flags)
+{
+    return Registry_Unload(interp, flags);
+}
+#endif
 
 /*
  *----------------------------------------------------------------------
@@ -286,11 +296,11 @@ static int
 RegistryObjCmd(
     void *dummy,	/* Not used. */
     Tcl_Interp *interp,		/* Current interpreter. */
-    int objc,			/* Number of arguments. */
+    Tcl_Size objc,			/* Number of arguments. */
     Tcl_Obj *const objv[])	/* Argument values. */
 {
-    int n = 1;
-    int index, argc;
+    Tcl_Size n = 1, argc;
+    int index;
     REGSAM mode = 0;
     const char *errString = NULL;
 
@@ -377,7 +387,7 @@ RegistryObjCmd(
 	     */
 
 	    mode |= KEY_ALL_ACCESS;
-	    if (OpenKey(interp, objv[n], mode, 1, &key) != TCL_OK) {
+	    if (OpenKey(interp, objv[n], mode, REG_CREATE, &key) != TCL_OK) {
 		return TCL_ERROR;
 	    }
 	    RegCloseKey(key);
@@ -438,13 +448,14 @@ DeleteKey(
     DWORD result;
     Tcl_DString buf;
     REGSAM saveMode = mode;
+    Tcl_Size len;
 
     /*
      * Find the parent of the key being deleted and open it.
      */
 
-    keyName = Tcl_GetString(keyNameObj);
-    buffer = (char *)Tcl_Alloc(keyNameObj->length + 1);
+    keyName = Tcl_GetStringFromObj(keyNameObj, &len);
+    buffer = (char *)Tcl_Alloc(len + 1);
     strcpy(buffer, keyName);
 
     if (ParseKeyName(interp, buffer, &hostName, &rootKey,
@@ -456,7 +467,7 @@ DeleteKey(
     if (*keyName == '\0') {
 	Tcl_SetObjResult(interp,
 		Tcl_NewStringObj("bad key: cannot delete root keys", -1));
-	Tcl_SetErrorCode(interp, "WIN_REG", "DEL_ROOT_KEY", NULL);
+	Tcl_SetErrorCode(interp, "WIN_REG", "DEL_ROOT_KEY", (char *)NULL);
 	Tcl_Free(buffer);
 	return TCL_ERROR;
     }
@@ -532,6 +543,7 @@ DeleteValue(
     char *valueName;
     DWORD result;
     Tcl_DString ds;
+    Tcl_Size len;
 
     /*
      * Attempt to open the key for deletion.
@@ -542,9 +554,9 @@ DeleteValue(
 	return TCL_ERROR;
     }
 
-    valueName = Tcl_GetString(valueNameObj);
+    valueName = Tcl_GetStringFromObj(valueNameObj, &len);
     Tcl_DStringInit(&ds);
-    Tcl_UtfToWCharDString(valueName, valueNameObj->length, &ds);
+    Tcl_UtfToWCharDString(valueName, len, &ds);
     result = RegDeleteValueW(key, (const WCHAR *)Tcl_DStringValue(&ds));
     Tcl_DStringFree(&ds);
     if (result != ERROR_SUCCESS) {
@@ -685,6 +697,7 @@ GetType(
     Tcl_DString ds;
     const char *valueName;
     const WCHAR *nativeValue;
+    Tcl_Size len;
 
     /*
      * Attempt to open the key for reading.
@@ -699,9 +712,9 @@ GetType(
      * Get the type of the value.
      */
 
-    valueName = Tcl_GetString(valueNameObj);
+    valueName = Tcl_GetStringFromObj(valueNameObj, &len);
     Tcl_DStringInit(&ds);
-    nativeValue = Tcl_UtfToWCharDString(valueName, valueNameObj->length, &ds);
+    nativeValue = Tcl_UtfToWCharDString(valueName, len, &ds);
     result = RegQueryValueExW(key, nativeValue, NULL, &type,
 	    NULL, NULL);
     Tcl_DStringFree(&ds);
@@ -757,6 +770,7 @@ GetValue(
     const WCHAR *nativeValue;
     DWORD result, length, type;
     Tcl_DString data, buf;
+    Tcl_Size len;
 
     /*
      * Attempt to open the key for reading.
@@ -773,7 +787,7 @@ GetValue(
      * length to TCL_DSTRING_STATIC_SIZE, but this should be safer if the
      * implementation of Dstrings changes.
      *
-     * This allows short values to be read from the registy in one call.
+     * This allows short values to be read from the registry in one call.
      * Longer values need a second call with an expanded DString.
      */
 
@@ -781,9 +795,9 @@ GetValue(
     Tcl_DStringSetLength(&data, TCL_DSTRING_STATIC_SIZE - 1);
     length = TCL_DSTRING_STATIC_SIZE/sizeof(WCHAR) - 1;
 
-    valueName = Tcl_GetString(valueNameObj);
+    valueName = Tcl_GetStringFromObj(valueNameObj, &len);
     Tcl_DStringInit(&buf);
-    nativeValue = Tcl_UtfToWCharDString(valueName, valueNameObj->length, &buf);
+    nativeValue = Tcl_UtfToWCharDString(valueName, len, &buf);
 
     result = RegQueryValueExW(key, nativeValue, NULL, &type,
 	    (BYTE *) Tcl_DStringValue(&data), &length);
@@ -795,7 +809,7 @@ GetValue(
 	 */
 
 	length = Tcl_DStringLength(&data) * (2 / sizeof(WCHAR));
-	Tcl_DStringSetLength(&data, (int) length * sizeof(WCHAR));
+	Tcl_DStringSetLength(&data, length * sizeof(WCHAR));
 	result = RegQueryValueExW(key, nativeValue,
 		NULL, &type, (BYTE *) Tcl_DStringValue(&data), &length);
     }
@@ -840,7 +854,7 @@ GetValue(
 		    Tcl_NewStringObj(Tcl_DStringValue(&buf),
 			    Tcl_DStringLength(&buf)));
 
-	    while (*wp++ != 0) {/* empty body */}
+	    while (*wp++ != 0); /* empty loop body */
 	    p = (char *) wp;
 	    Tcl_DStringFree(&buf);
 	}
@@ -856,7 +870,7 @@ GetValue(
 	 */
 
 	Tcl_SetObjResult(interp, Tcl_NewByteArrayObj(
-		(BYTE *) Tcl_DStringValue(&data), (int) length));
+		(BYTE *) Tcl_DStringValue(&data), length));
     }
     Tcl_DStringFree(&data);
     return result;
@@ -867,7 +881,7 @@ GetValue(
  *
  * GetValueNames --
  *
- *	This function enumerates the values of the a given key. If the
+ *	This function enumerates the values of the given key. If the
  *	optional pattern is supplied, then only value names that match the
  *	pattern will be returned.
  *
@@ -905,7 +919,7 @@ GetValueNames(
 
     resultPtr = Tcl_NewObj();
     Tcl_DStringInit(&buffer);
-    Tcl_DStringSetLength(&buffer, (int) (MAX_KEY_LENGTH * sizeof(WCHAR)));
+    Tcl_DStringSetLength(&buffer, MAX_KEY_LENGTH * sizeof(WCHAR));
     index = 0;
     result = TCL_OK;
 
@@ -924,7 +938,6 @@ GetValueNames(
     size = MAX_KEY_LENGTH;
     while (RegEnumValueW(key,index, (WCHAR *)Tcl_DStringValue(&buffer),
 	    &size, NULL, NULL, NULL, NULL) == ERROR_SUCCESS) {
-
 	Tcl_DStringInit(&ds);
 	Tcl_WCharToUtfDString((const WCHAR *)Tcl_DStringValue(&buffer), size, &ds);
 	name = Tcl_DStringValue(&ds);
@@ -975,9 +988,10 @@ OpenKey(
     char *keyName, *buffer, *hostName;
     HKEY rootKey;
     DWORD result;
+    Tcl_Size len;
 
-    keyName = Tcl_GetString(keyNameObj);
-    buffer = (char *)Tcl_Alloc(keyNameObj->length + 1);
+    keyName = Tcl_GetStringFromObj(keyNameObj, &len);
+    buffer = (char *)Tcl_Alloc(len + 1);
     strcpy(buffer, keyName);
 
     result = ParseKeyName(interp, buffer, &hostName, &rootKey, &keyName);
@@ -1002,7 +1016,7 @@ OpenKey(
  *
  * OpenSubKey --
  *
- *	This function opens a given subkey of a root key on the specified
+ *	Opens a given subkey of the given root key on the specified
  *	host.
  *
  * Results:
@@ -1087,7 +1101,7 @@ OpenSubKey(
  *
  * ParseKeyName --
  *
- *	This function parses a key name into the host, root, and subkey parts.
+ *	Parses a key name into the host, root, and subkey parts.
  *
  * Results:
  *	The pointers to the start of the host and subkey names are returned in
@@ -1133,7 +1147,7 @@ ParseKeyName(
     if (!rootName) {
 	Tcl_SetObjResult(interp, Tcl_ObjPrintf(
 		"bad key \"%s\": must start with a valid root", name));
-	Tcl_SetErrorCode(interp, "WIN_REG", "NO_ROOT_KEY", NULL);
+	Tcl_SetErrorCode(interp, "WIN_REG", "NO_ROOT_KEY", (char *)NULL);
 	return TCL_ERROR;
     }
 
@@ -1211,7 +1225,7 @@ RecursiveDeleteKey(
     }
 
     Tcl_DStringInit(&subkey);
-    Tcl_DStringSetLength(&subkey, (int) (MAX_KEY_LENGTH * sizeof(WCHAR)));
+    Tcl_DStringSetLength(&subkey, MAX_KEY_LENGTH * sizeof(WCHAR));
 
     mode = saveMode;
     while (result == ERROR_SUCCESS) {
@@ -1285,6 +1299,7 @@ SetValue(
     HKEY key;
     const char *valueName;
     Tcl_DString nameBuf;
+    Tcl_Size len;
 
     if (typeObj == NULL) {
 	type = REG_SZ;
@@ -1296,13 +1311,13 @@ SetValue(
 	Tcl_ResetResult(interp);
     }
     mode |= KEY_ALL_ACCESS;
-    if (OpenKey(interp, keyNameObj, mode, 1, &key) != TCL_OK) {
+    if (OpenKey(interp, keyNameObj, mode, REG_CREATE, &key) != TCL_OK) {
 	return TCL_ERROR;
     }
 
-    valueName = Tcl_GetString(valueNameObj);
+    valueName = Tcl_GetStringFromObj(valueNameObj, &len);
     Tcl_DStringInit(&nameBuf);
-    valueName = (char *) Tcl_UtfToWCharDString(valueName, valueNameObj->length, &nameBuf);
+    valueName = (char *) Tcl_UtfToWCharDString(valueName, len, &nameBuf);
 
     if (type == REG_DWORD || type == REG_DWORD_BIG_ENDIAN) {
 	int value;
@@ -1318,7 +1333,7 @@ SetValue(
 		(DWORD) type, (BYTE *) &value, sizeof(DWORD));
     } else if (type == REG_MULTI_SZ) {
 	Tcl_DString data, buf;
-	int objc, i;
+	Tcl_Size objc, i;
 	Tcl_Obj **objv;
 
 	if (Tcl_ListObjGetElements(interp, dataObj, &objc, &objv) != TCL_OK) {
@@ -1335,9 +1350,9 @@ SetValue(
 
 	Tcl_DStringInit(&data);
 	for (i = 0; i < objc; i++) {
-	    const char *bytes = Tcl_GetString(objv[i]);
+	    const char *bytes = Tcl_GetStringFromObj(objv[i], &len);
 
-	    Tcl_DStringAppend(&data, bytes, objv[i]->length);
+	    Tcl_DStringAppend(&data, bytes, len);
 
 	    /*
 	     * Add a null character to separate this value from the next.
@@ -1356,10 +1371,10 @@ SetValue(
 	Tcl_DStringFree(&buf);
     } else if (type == REG_SZ || type == REG_EXPAND_SZ) {
 	Tcl_DString buf;
-	const char *data = Tcl_GetString(dataObj);
+	const char *data = Tcl_GetStringFromObj(dataObj, &len);
 
 	Tcl_DStringInit(&buf);
-	data = (char *) Tcl_UtfToWCharDString(data, dataObj->length, &buf);
+	data = (char *) Tcl_UtfToWCharDString(data, len, &buf);
 
 	/*
 	 * Include the null in the length, padding if needed for WCHAR.
@@ -1372,13 +1387,13 @@ SetValue(
 	Tcl_DStringFree(&buf);
     } else {
 	BYTE *data;
-	size_t bytelength;
+	Tcl_Size bytelength;
 
 	/*
 	 * Store binary data in the registry.
 	 */
 
-	data = (BYTE *) getByteArrayFromObj(dataObj, &bytelength);
+	data = (BYTE *) Tcl_GetByteArrayFromObj(dataObj, &bytelength);
 	result = RegSetValueExW(key, (WCHAR *) valueName, 0,
 		(DWORD) type, data, (DWORD) bytelength);
     }
@@ -1415,21 +1430,20 @@ SetValue(
 static int
 BroadcastValue(
     Tcl_Interp *interp,		/* Current interpreter. */
-    int objc,			/* Number of arguments. */
+    Tcl_Size objc,			/* Number of arguments. */
     Tcl_Obj *const objv[])	/* Argument values. */
 {
     LRESULT result;
     DWORD_PTR sendResult;
     int timeout = 3000;
-    size_t len;
+    Tcl_Size len;
     const char *str;
     Tcl_Obj *objPtr;
     WCHAR *wstr;
     Tcl_DString ds;
 
     if (objc == 3) {
-	str = Tcl_GetString(objv[1]);
-	len = objv[1]->length;
+	str = Tcl_GetStringFromObj(objv[1], &len);
 	if ((len < 2) || (*str != '-') || strncmp(str, "-timeout", len)) {
 	    return TCL_BREAK;
 	}
@@ -1438,9 +1452,9 @@ BroadcastValue(
 	}
     }
 
-    str = Tcl_GetString(objv[0]);
+    str = Tcl_GetStringFromObj(objv[0], &len);
     Tcl_DStringInit(&ds);
-    wstr = Tcl_UtfToWCharDString(str, objv[0]->length, &ds);
+    wstr = Tcl_UtfToWCharDString(str, len, &ds);
     if (Tcl_DStringLength(&ds) == 0) {
 	wstr = NULL;
     }
@@ -1466,7 +1480,7 @@ BroadcastValue(
  *
  * AppendSystemError --
  *
- *	This routine formats a Windows system error message and places it into
+ *	Formats a Windows system error message and places it into
  *	the interpreter result.
  *
  * Results:
@@ -1498,7 +1512,7 @@ AppendSystemError(
 	    MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (WCHAR *) tMsgPtrPtr,
 	    0, NULL);
     if (length == 0) {
-	sprintf(msgBuf, "unknown error: %ld", error);
+	snprintf(msgBuf, sizeof(msgBuf), "unknown error: %ld", error);
 	msg = msgBuf;
     } else {
 	char *msgPtr;
@@ -1524,8 +1538,8 @@ AppendSystemError(
 	msg = msgPtr;
     }
 
-    sprintf(id, "%ld", error);
-    Tcl_SetErrorCode(interp, "WINDOWS", id, msg, NULL);
+    snprintf(id, sizeof(id), "%ld", error);
+    Tcl_SetErrorCode(interp, "WINDOWS", id, msg, (char *)NULL);
     Tcl_AppendToObj(resultPtr, msg, length);
     Tcl_SetObjResult(interp, resultPtr);
 
@@ -1539,7 +1553,7 @@ AppendSystemError(
  *
  * ConvertDWORD --
  *
- *	This function determines whether a DWORD needs to be byte swapped, and
+ *	Determines whether a DWORD needs to be byte swapped, and
  *	returns the appropriately swapped value.
  *
  * Results:
